@@ -38,6 +38,10 @@
 #include "uthash.h"
 #include <pthread.h>
 
+#ifdef HAVE_LIBPCRE
+  #include <pcre.h>
+#endif
+
 #ifdef HAVE_LIBSSL
   #include <openssl/ssl.h>
   #include <openssl/err.h>
@@ -208,7 +212,7 @@ int medusaConnectInternal(unsigned long nHost, int nPort, int nProtocol, int nTy
             if (nOpt != 0) 
             { 
               // Socket is not valid - connection failed
-              writeVerbose(VB_GENERAL, "Unable to connect (invalid socket): unreachable destination"); 
+              writeVerbose(VB_GENERAL, "Unable to connect (invalid socket): unreachable destination - %s", inet_ntop(AF_INET, &target.sin_addr, out, sizeof(out)));
               return -1; 
             }
             
@@ -306,6 +310,8 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
   struct SSLSOCKETINFO *s;
   SSL *ssl = NULL;
   SSL_CTX *sslContext = NULL;
+  
+  pthread_mutex_lock(&ptmSSLMutex);
 
   SSL_load_error_strings();
   SSLeay_add_ssl_algorithms();
@@ -316,6 +322,8 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
     sslContext = SSL_CTX_new(SSLv2_client_method());
   else if (pParams->nSSLVersion == 3)
     sslContext = SSL_CTX_new(SSLv3_client_method());
+  else if (pParams->nSSLVersion == (float)3.1)
+    sslContext = SSL_CTX_new(TLSv1_client_method());
   else
     sslContext = SSL_CTX_new(SSLv23_client_method());
 
@@ -363,8 +371,9 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
   s->nUseSSL = 1;
   s->ssl = ssl;
   s->sslContext = sslContext;
-  pthread_mutex_lock(&ptmSSLMutex);
+  
   HASH_ADD_INT( psSSLSocketInfo, id, s ); 
+  
   pthread_mutex_unlock(&ptmSSLMutex);
 
   return hSocket;
@@ -391,7 +400,7 @@ int medusaReceiveInternal(int socket, char *buf, int length)
         switch(err)
         {
           case SSL_ERROR_ZERO_RETURN:
-            writeError(ERR_ERROR, "The TLS/SSL connection has been closed.");
+            writeError(ERR_DEBUG, "The TLS/SSL connection has been closed.");
             break;
           case SSL_ERROR_SSL:
             writeError(ERR_ERROR, "A failure in the SSL library occurred, usually a protocol error.");
@@ -420,7 +429,26 @@ int medusaReceiveInternal(int socket, char *buf, int length)
 */
 char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsToSpaces, int nReceiveDelay1, int nReceiveDelay2)
 {
-  const unsigned int BUFFER_SIZE = 300;
+  /*
+    When receiving UDP packets, we need to be mindful of packets exceeding our default buffer size.
+    Since we configure our UDP socket as SOCK_DGRAM, any packets beyond the buffer size will be 
+    discarded following a recv() call. UDP messages, or datagrams, should have the entire message 
+    within a single packet. The actual maximum size of this packet is unknown, however, due to a 
+    number of variables (see reference below). For our purposes, we are going to use the value
+    of 576.
+
+    http://www.uic.rsu.ru/doc/inet/tcp_stevens/ip_inter.htm#3_2
+
+    Although it's possible to send a 65535-byte IP datagram, most link layers will fragment this. 
+    Furthermore, a host is not required to receive a datagram larger than 576 bytes. TCP divides 
+    the user's data into pieces, so this limit normally doesn't affect TCP. With UDP we'll 
+    encounter numerous applications in later chapters (RIP, TFTP, BOOTP, the DNS, and SNMP) that 
+    limit themselves to 512 bytes of user data, to stay below this 576-byte limit. Realistically, 
+    however, most implementations today (especially those that support the Network File System, 
+    NFS) allow for just over 8192-byte IP datagrams. 
+  */
+  const unsigned int BUFFER_SIZE = 576;
+  
   char *szBufReceive, *szBufReceiveTmp;
   int nBufReceive = 0, nBufReceiveTmp = 0, BufReceiveIndex = 0;
   int bSocketStatus = 0;
@@ -602,16 +630,19 @@ int medusaDisconnect(int hSocket)
   if (hSocket <= 0)
     return -1;
 
+  pthread_mutex_lock(&ptmSSLMutex);
+  
   /* Remove socket's SSL informational structure (if it exists) */
   HASH_FIND_INT( psSSLSocketInfo, &hSocket, s );
   if (s != NULL)
   {
-    pthread_mutex_lock(&ptmSSLMutex);
     HASH_DEL( psSSLSocketInfo, s );
-    pthread_mutex_unlock(&ptmSSLMutex);
   }
 
   close(hSocket);
+
+  pthread_mutex_unlock(&ptmSSLMutex);
+
   writeError(ERR_DEBUG, "Disconnect successful");
   return -1;
 #else
@@ -695,6 +726,104 @@ char* medusaReceiveLineDelay(int socket, int* nBufferSize, int nReceiveDelay1, i
   return medusaReceiveDataInternal(socket, nBufferSize, 1, nReceiveDelay1, nReceiveDelay2);
 }
 
+
+/*
+  Receive function which uses regular expressions to determine whether we read 
+  all the data we're intending to. The goal is to address the issue of varying 
+  network speeds of servers. We don't want to retrieve only the first few bytes
+  of a response and then start responding before the remote end is finished.
+
+  The function will recheck the socket 5 times before giving up finding a match.
+  Each recheck uses a larger timeout value.
+*/
+#ifdef HAVE_LIBPCRE
+int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufReceive, const char* regex)
+{
+  char *szBufReceiveTmp = NULL;
+  int nBufReceiveTmp = 0;
+  pcre *re = NULL;
+  const char *error = NULL;
+  int erroffset = 0;
+  int rc = 0; 
+  int nAttempt = 1;
+  const unsigned int BUFFER_SIZE = 576;
+
+  writeError(ERR_DEBUG, "PCRE regural expession: \"%s\"", regex);
+  /* PCRE_DOTALL - a dot metacharater in the pattern matches all characters, 
+     including those that indicate newline */
+  re = pcre_compile(regex, PCRE_CASELESS|PCRE_DOTALL, &error, &erroffset, NULL);
+  if (re == NULL)
+  {
+    writeError(ERR_ERROR, "PCRE compilation failed at offset %d: %s\n", erroffset, error);
+    return FAILURE;
+  }  
+
+  *szBufReceive = medusaReceiveDataInternal(hSocket, nBufReceive, 0, READ_WAIT_TIME, 0);
+
+  do
+  {
+    rc = pcre_exec(re, NULL, *szBufReceive, *nBufReceive, 0, 0, NULL, 0); 
+    if (rc < 0)
+    {
+      writeError(ERR_DEBUG, "Failed to match regex. Checking for additional data.");
+
+      /* there more be more data waiting for us... */
+      if (medusaDataReadyTimed(hSocket, 0, 20000 * nAttempt) > 0)
+      {
+        szBufReceiveTmp = malloc(BUFFER_SIZE + 1);
+        memset(szBufReceiveTmp, 0, BUFFER_SIZE + 1);
+
+        nBufReceiveTmp = medusaReceive(hSocket, szBufReceiveTmp, BUFFER_SIZE);
+        if (nBufReceiveTmp <= 0)
+        {
+          writeError(ERR_DEBUG, "Data receive: No additional data.");
+          free(szBufReceiveTmp);
+          break;
+        }
+ 
+        if (*nBufReceive + nBufReceiveTmp > BUFFER_SIZE) 
+        {
+          writeError(ERR_DEBUG, "Additional data received. Increasing receive buffer %d bytes to %d.", nBufReceiveTmp, *nBufReceive + nBufReceiveTmp + 1);
+          *szBufReceive = realloc(szBufReceive, *nBufReceive + nBufReceiveTmp + 1);
+        }
+
+        memcpy(*szBufReceive + *nBufReceive, szBufReceiveTmp, nBufReceiveTmp);
+        *nBufReceive += nBufReceiveTmp;
+
+        nBufReceiveTmp = 0;
+        free(szBufReceiveTmp);
+      }
+      else
+      {
+        /* no additional data found... let's check it a few times */
+        writeError(ERR_DEBUG, "No additional data found (attempt %d/5)", nAttempt);
+        nAttempt++;
+      }
+    }
+    else
+    {
+      writeError(ERR_DEBUG, "Successfully matched regex.");
+      return SUCCESS;
+    }
+  } while (nAttempt <= 5);
+
+  writeError(ERR_ERROR, "Failed to match regex pattern within server's response.");
+  return FAILURE;
+}
+#else
+int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufReceive, const char* regex)
+{
+  writeError(ERR_ERROR, "Medusa compiled without PCRE support. Data receiving may be less reliable.");
+
+  *szBufReceive = medusaReceiveDataInternal(hSocket, nBufReceive, 0, READ_WAIT_TIME, 0);
+
+  if (*szBufReceive == NULL)
+    return FAILURE;
+  else
+    return SUCCESS;
+}
+#endif
+
 int medusaSend(int socket, char *buf, int size, int options)
 {
   char debugbuf[size + 1];
