@@ -74,10 +74,10 @@ void writeVerbose(int iLevel, char *pMsg, ...) {
         fprintf(stdout, "ACCOUNT FOUND: %s\n", bufOut);
         
         if (pOutputFile != NULL) {
-          pthread_mutex_lock(&ptmMutex);
+          pthread_mutex_lock(&ptmFileMutex);
           fprintf(pOutputFile, "ACCOUNT FOUND: %s\n", buf);
           fflush(pOutputFile);
-          pthread_mutex_unlock(&ptmMutex);
+          pthread_mutex_unlock(&ptmFileMutex);
         }
 
         va_end(ap);
@@ -100,10 +100,10 @@ void writeVerbose(int iLevel, char *pMsg, ...) {
         break;
       case VB_NONE_FILE:
         if (pOutputFile != NULL) {
-          pthread_mutex_lock(&ptmMutex);
+          pthread_mutex_lock(&ptmFileMutex);
           fprintf(pOutputFile, "%s", bufOut);
           fflush(pOutputFile);
-          pthread_mutex_unlock(&ptmMutex);
+          pthread_mutex_unlock(&ptmFileMutex);
         }
         
         va_end(ap);
