@@ -24,6 +24,7 @@
 #define _MEDUSA_H
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <pthread.h>
@@ -31,9 +32,11 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#include <math.h>
 
 #include "medusa-trace.h"
 #include "medusa-net.h"
+#include "medusa-thread-pool.h"
 
 #ifdef HAVE_CONFIG_H
   #include <config.h>
@@ -55,14 +58,33 @@
 
 /* GLOBAL VARIABLES */
 FILE *pOutputFile;
-pthread_mutex_t ptmMutex;
+pthread_mutex_t ptmFileMutex;
 int iVerboseLevel;      // Global control over general message verbosity
 int iErrorLevel;        // Global control over error debugging verbosity
 
-
 //#define MAX_BUF (16 * 1024)
 #define MAX_BUF 16384 
-#define NUM_THREADS 6
+
+/* Older Solaris doesn't seem to define INADDR_NONE */
+#ifndef INADDR_NONE
+  #define INADDR_NONE ((unsigned long) -1
+#endif
+
+/* Cygwin doesn't seem to define INET_ADDRSTRLEN */
+#ifndef INET_ADDRSTRLEN
+  #define INET_ADDRSTRLEN 16
+#endif
+
+// Number of seconds that idle threads can linger before exiting, when no tasks 
+// come in. The idle threads can only exit if they are extra threads, above the 
+// number of minimum threads.
+#define POOL_THREAD_LINGER 1
+
+#define FREE(x) \
+        if (x != NULL) { \
+           free(x); \
+           x = NULL; \
+        }
 
 #define L_UNSET 0
 #define L_SINGLE 1
@@ -75,12 +97,14 @@ typedef struct __sPass {
   char *pPass;
 } sPass;
 
-/* Used in __sUser to defined progress of an individual username audit */
+/* Used in __sUser to define progress of an individual username audit */
+#define PL_UNSET 0
 #define PL_NULL 1
 #define PL_USERNAME 2
 #define PL_LOCAL 3
 #define PL_GLOBAL 4
 #define PL_DONE 5
+#define PASS_AUDIT_COMPLETE 6 
 
 typedef struct __sUser {
   struct __sUser *psUserNext;
@@ -89,26 +113,18 @@ typedef struct __sUser {
   struct __sPass *psPassCurrent;
   struct __sPass *psPassPrevTmp;
   char *pPass;
-  char *pPassIndex;
-  char *pValidPass;
   int iPassCnt;
   int iLoginsDone;
   int iPassStatus;
   int iId;
-
-  pthread_mutex_t ptmMutex;
 } sUser;
 
-/* Used in __sHost to defined progress of an individual host audit */
-#define UL_NULL 1
-#define UL_ACTIVE 2
+/* Used in __sHost to define progress of the audit of the host's users */
+#define UL_UNSET 0
+#define UL_NORMAL 1
+#define UL_MISSED 2
 #define UL_DONE 3
-#define UL_SUCCESS 4
-
-/* Used in __sHost to defined progress of host audit */
-#define HL_NULL 1;
-#define HL_ACTIVE 2;
-#define HL_DONE 3;
+#define UL_ERROR 4
 
 typedef struct __sHost {
   struct __sHost *psHostNext;
@@ -118,51 +134,45 @@ typedef struct __sHost {
   int iTimeout;           // Number of seconds to wait before a connection times out
   int iRetryWait;         // Number of seconds to wait between retries
   int iRetries;           // Number of retries to attempt
-  char *pUserIndex;
   sUser *psUser;
+  sUser *psUserCurrent;
   sUser *psUserPrevTmp;
   int iUserCnt;
   int iUserPassCnt;
   int iUsersDone;        // number of users tested
   int iUserStatus;
-  int iHostStatus;
   int iId;
 } sHost;
 
-#define SRV_IDLE 1
-#define SRV_ACTIVE 2
-#define SRV_DONE 3
+/* Used in __sCredentialSet to relay information to module regarding user */
+#define CREDENTIAL_SAME_USER 1
+#define CREDENTIAL_NEW_USER 2
+#define CREDENTIAL_DONE 3
 
-#define SRV_LOGIN_IN_PROGRESS 4
-#define SRV_LOGIN_DONE 5
+typedef struct __sCredentialSet {
+  struct __sCredentialSet *psCredentialSetNext;
+  struct __sUser *psUser;
+  char *pPass;
+  int iStatus;
+} sCredentialSet;
 
 typedef struct __sServer {
   struct __sAudit *psAudit;
   struct __sHost *psHost;
-
   char *pHostIP;
-  
-  char *pUserSuccess;
-  char *pPassSuccess;
-
-  //int iExitOnFound;
   int iValidPairFound;
-  //int iStartTime;
-  //int iEndTime;
-  //int iAccountsTested;
-  int iStatus;
-
   int iId;
+  int iLoginCnt;          // total number of logins performed concurrently against specific server
   int iLoginsDone;       // number of logins performed by all threads under this server
+  
+  sCredentialSet *psCredentialSetMissed;
+  sCredentialSet *psCredentialSetMissedCurrent;
+  sCredentialSet *psCredentialSetMissedTail;
+  int iCredentialsMissed;
+
   pthread_mutex_t ptmMutex;
 } sServer;
 
-#define LOGIN_IDLE 1
-#define LOGIN_ACTIVE 2
-#define LOGIN_DONE 3
-#define LOGIN_FAILED 4
-#define LOGIN_DISABLED 5
-
 #define LOGIN_RESULT_UNKNOWN 1
 #define LOGIN_RESULT_SUCCESS 2
 #define LOGIN_RESULT_FAIL 3
@@ -171,7 +181,6 @@ typedef struct __sServer {
 typedef struct __sLogin {
   struct __sServer *psServer;
   struct __sUser *psUser;
-  int iStatus;
   int iResult;
   char *pErrorMsg;
   int iId;
@@ -190,12 +199,15 @@ typedef struct __sLogin {
 #define PARALLEL_LOGINS_USER 1
 #define PARALLEL_LOGINS_PASSWORD 2
 
+#define AUDIT_ABORT 1
+
 typedef struct __sAudit {
   char *pOptHost;         // user specified host or host file
   char *pOptUser;         // user specified username or username file
   char *pOptPass;         // user specified password or password file
   char *pOptCombo;        // user specified combo host/username/password file
   char *pOptOutput;       // user specified output file
+  char *pOptResume;       // user specified resume command
 
   char *pModuleName;      // current module name
 
@@ -214,7 +226,6 @@ typedef struct __sAudit {
   int iComboCnt;          // total number of entries in combo file
   int iServerCnt;         // total number of hosts scanned concurrently
   int iLoginCnt;          // total number of logins performed concurrently
-  int iResume;		  // will resume be enabled and at what host #
 
   int iHostsDone;         // number of hosts tested
 
@@ -239,8 +250,13 @@ typedef struct __sAudit {
   int iFoundPairExitFlag;     /* When a valid login pair is found, end scan of host or of complete audit */
   int iParallelLoginFlag;     /* Parallel logins by user or password */
   int iValidPairFound;
-  
+  int iStatus;                /* Flag to indicate to threads that audit is aborting */ 
+ 
   sHost *psHostRoot;
+ 
+  thr_pool_t *server_pool;
+ 
+  pthread_mutex_t ptmMutex;
 } sAudit;
 
 typedef struct __sModuleStart
@@ -251,7 +267,5 @@ typedef struct __sModuleStart
   char**  argv;  
 } sModuleStart;
 
-char* getNextPass(sAudit *_psAudit, sUser *_psUser);
-
 #define _MEDUSA_H
 #endif
