@@ -2,7 +2,7 @@
 **   FTP Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 pMonkey
+**    Copyright (C) 2009 pMonkey
 **    pMonkey <pmonkey@foofus.net>
 **
 **    CHANGE LOG
@@ -36,18 +36,12 @@
 #define MODULE_NAME    "ftp.mod"
 #define MODULE_AUTHOR  "pMonkey <pmonkey@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for FTP/FTPS sessions"
-#define MODULE_VERSION    "1.3.0"
-#define MODULE_VERSION_SVN "$Id: ftp.c 819 2007-09-20 22:05:45Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: ftp.c 1296 2010-02-04 19:19:36Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_FTP  21
 #define PORT_FTPS 990
 
@@ -65,7 +59,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -182,7 +177,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
@@ -193,10 +189,27 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
   int nReceiveBufferSize = 0;
   int nStatus = FAILURE;
   int nBannerStatus = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   _psSessionData->params = malloc( sizeof(sConnectParams) );
   memset(_psSessionData->params, 0, sizeof(sConnectParams));
   
@@ -206,8 +219,10 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
     _psSessionData->params->nPort = PORT_FTPS;
   }
   else
+  {
     _psSessionData->params->nPort = PORT_FTP;
-  
+  }  
+
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     _psSessionData->params->nPort = psLogin->psServer->psAudit->iPortOverride;
   else if (psLogin->psServer->psHost->iUseSSL > 0)
@@ -215,24 +230,9 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 
   initConnectionParams(psLogin, _psSessionData->params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         // Already have an open socket - close it
@@ -248,8 +248,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, _psSessionData->params->nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          FREE(_psSessionData);
           return FAILURE;
         }
 
@@ -281,6 +279,9 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
             writeError(ERR_DEBUG_MODULE, "[%s] Server sent '220' code.", MODULE_NAME);
             nStatus = SUCCESS;
             FREE(bufReceive);
+            
+            // fizzgig: We may need to receive more banner data here
+            //continue;
             break;
           }
 
@@ -299,7 +300,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_DEBUG_MODULE, "[%s] Server did not respond with '220' code. Exiting...", MODULE_NAME);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           nState = MSTATE_EXITING;
           return FAILURE;
           break;
@@ -316,7 +316,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           if (initAuthSSL(hSocket, _psSessionData) == FAILURE)
           {
             psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-            psLogin->iStatus = LOGIN_FAILED;
             nState = MSTATE_EXITING;
           }
         }
@@ -328,9 +327,31 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
            until that disconnect happens. At that point the connection should be reestablished. */
         if ( medusaCheckSocket(hSocket) )
         {
-          nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
+          nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
           if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-            pPass = getNextPass(psLogin->psServer->psAudit, user);
+          {
+            if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+            {
+              writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else
+            {
+              if (psCredSet->iStatus == CREDENTIAL_DONE)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+                nState = MSTATE_EXITING;
+              }
+              else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+                nState = MSTATE_NEW;
+              }
+              else
+                writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+            }
+          }
         }
         else
         {
@@ -347,7 +368,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -355,14 +376,11 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
-        FREE(_psSessionData);
         return FAILURE;
     }  
   }
  
-  psLogin->iStatus = LOGIN_DONE;
-  FREE(_psSessionData);
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -426,12 +444,13 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
   if (bufReceive == NULL)
   {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data. Server may have dropped connection due to lack of encryption. Enabling the EXPLICIT mode may help.", MODULE_NAME);
     return FAILURE;
   }
 
   /* FTP service may be configured to require protected authentication for specific users */
   if ( (strstr(bufReceive, "530 Non-anonymous sessions must use encryption.") != NULL) ||
+       (strstr(bufReceive, "331 Non-anonymous sessions must use encryption.") != NULL) || 
        (strstr(bufReceive, "331 Rejected--secure connection required") != NULL) )
   {
     writeError(ERR_NOTICE, "[%s] FTP server (%s) appears to require SSL for specified user.", MODULE_NAME, (*psLogin)->psServer->pHostIP);
