@@ -1,6 +1,6 @@
 /***************************************************************************
  *   http.c                                                                *
- *   Copyright (C) 2006 by fizzgig                                         *
+ *   Copyright (C) 2009 by fizzgig                                         *
  *   fizzgig@foofus.net                                                    *
  *                                                                         *
  *   Implementation of a http brute force module for                       *
@@ -36,30 +36,23 @@
  ***************************************************************************/
 
 #include "module.h"
-#include <math.h>
 
 #define	MODULE_NAME		"http.mod"
 #define MODULE_AUTHOR  "fizzgig <fizzgig@foofus.net>"
 #define	MODULE_SUMMARY_USAGE	"Brute force module for HTTP"
-#define MODULE_VERSION		"1.3.0"
-#define MODULE_VERSION_SVN "$Id: http.c 980 2008-10-01 20:22:51Z jmk $"
+#define MODULE_VERSION		"2.0"
+#define MODULE_VERSION_SVN "$Id: http.c 1259 2009-11-16 04:19:46Z jmk $"
 #define MODULE_SUMMARY_FORMAT	"%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #ifdef HAVE_LIBSSL
 
 #include "ntlm.h"
 #include "http-digest.h"
 
-#define HTTP_PORT 80
-#define HTTPS_PORT 443
+#define PORT_HTTP 80
+#define PORT_HTTPS 443
 
 #define AUTH_UNKNOWN 0
 #define AUTH_NONE 1
@@ -68,7 +61,7 @@
 #define AUTH_DIGEST 4
 
 typedef struct __MODULE_DATA {
-  //char *szDomain;
+  char *szDomain;
   char *szDir;
   char *szHostHeader;
   char *szUserAgent;
@@ -78,15 +71,16 @@ typedef struct __MODULE_DATA {
 // Tells us whether we are to continue processing or not
 enum HTTP_STATE
 {
-  HSTATE_NEW,
-  HSTATE_RUNNING,
-  HSTATE_EXITING
+  MSTATE_NEW,
+  MSTATE_RUNNING,
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
 int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort);
 int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort, char* szLogin, char* szPassword);
-int initHttp(_MODULE_DATA* _psSessionData, sLogin* login);
+int initModule(_MODULE_DATA* _psSessionData, sLogin* login);
 
 // Tell medusa how many parameters this module allows
 int getParamNumber()
@@ -120,6 +114,8 @@ void showUsage()
   writeVerbose(VB_NONE, "Available module options:");
   writeVerbose(VB_NONE, "  USER-AGENT:? (User-Agent. Default: Teh Forest Lobster.)");
   writeVerbose(VB_NONE, "  DIR:? (Target directory. Default \"/\"");
+  writeVerbose(VB_NONE, "  AUTH:? (Authentication Type (BASIC/DIGEST/NTLM). Default: automatic)");
+  writeVerbose(VB_NONE, "  DOMAIN:? [optional] (BASIC/NTLM)");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "Usage example: \"-M http -m USER-AGENT:\"g3rg3 gerg\" -m DIR:exchange/\"");
 }
@@ -130,13 +126,10 @@ int go(sLogin* logins, int argc, char *argv[])
   int i;
   char *strtok_ptr, *pOpt, *pOptTmp;
   _MODULE_DATA *psSessionData;
+
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  //psSessionData->szDomain = malloc(strlen("FOO") + 1);
-  //memset(psSessionData->szDomain, 0, strlen("FOO") + 1);
-  //sprintf(psSessionData->szDomain, "FOO"); 
- 
   if ( !( 0 <= argc <= 2) )
   {
     // Show usage information
@@ -146,7 +139,7 @@ int go(sLogin* logins, int argc, char *argv[])
   else
   {
     // Parameters are good - make module go now
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
  
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
@@ -184,6 +177,34 @@ int go(sLogin* logins, int argc, char *argv[])
         else
           writeError(ERR_WARNING, "Method USER-AGENT requires value to be set.");
       }
+      else if (strcmp(pOpt, "AUTH") == 0)
+      {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if (strcmp(pOpt, "BASIC") == 0)
+          psSessionData->nAuthType = AUTH_BASIC;
+        else if (strcmp(pOpt, "DIGEST") == 0)
+          psSessionData->nAuthType = AUTH_DIGEST;
+        else if (strcmp(pOpt, "NTLM") == 0)
+          psSessionData->nAuthType = AUTH_NTLM;
+        else
+          writeError(ERR_WARNING, "Invalid value for method AUTH.");
+      }
+      else if (strcmp(pOpt, "DOMAIN") == 0)
+      {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if ( pOpt )
+        {
+          psSessionData->szDomain = malloc(strlen(pOpt) + 1);
+          memset(psSessionData->szDomain, 0, strlen(pOpt) + 1);
+          strncpy((char *) psSessionData->szDomain, pOpt, strlen(pOpt));
+        }
+        else
+          writeError(ERR_WARNING, "Method DOMAIN requires value to be set.");
+      }
       else
       {
         writeError(ERR_WARNING, "Invalid method: %s.", pOpt);
@@ -192,136 +213,153 @@ int go(sLogin* logins, int argc, char *argv[])
       FREE(pOptTmp);
     }
 
-    initHttp(psSessionData, logins);
+    initModule(psSessionData, logins);
   }
 
+  FREE(psSessionData->szDir);
+  FREE(psSessionData->szUserAgent);
+  FREE(psSessionData->szDomain);
+  FREE(psSessionData);
   return SUCCESS;
 }
 
-int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
+int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
 {
   int hSocket = -1;
-  enum HTTP_STATE nState = HSTATE_NEW;
-  char* bufReceive;
-  int i = 0;
-  char *pPass;
-  int nPort, nBufLength = 0;
-  sUser* user = login->psUser;
+  enum HTTP_STATE nState = MSTATE_NEW;
+  int nBufLength = 0;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
-  
+ 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, _psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME, _psLogin->psServer->pHostIP);
+    nState = MSTATE_COMPLETE;
+  }
+ 
   memset(&params, 0, sizeof(sConnectParams));
-  if (login->psServer->psAudit->iPortOverride > 0)
-    params.nPort = login->psServer->psAudit->iPortOverride;
-  else if (login->psServer->psHost->iUseSSL > 0)
-    params.nPort = HTTPS_PORT;
+  if (_psLogin->psServer->psAudit->iPortOverride > 0)
+    params.nPort = _psLogin->psServer->psAudit->iPortOverride;
+  else if (_psLogin->psServer->psHost->iUseSSL > 0)
+    params.nPort = PORT_HTTPS;
   else
-    params.nPort = HTTP_PORT; 
-  initConnectionParams(login, &params);
+    params.nPort = PORT_HTTP; 
+  initConnectionParams(_psLogin, &params);
 
-  if (user != NULL)
+  /* Set request parameters */
+  if (!_psSessionData->szDir)
   {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s:%d, user: '%s'", MODULE_NAME, login->psServer->pHostIP, params.nPort, user->pUser);
-  }
-  else
-  {
-    writeError(ERR_DEBUG, "null user");
-  }
-  
-  pPass = getNextPass(login->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, login->psServer->pHostIP);
+    _psSessionData->szDir = malloc(1);
+    memset(_psSessionData->szDir, 0, 1);
   }
 
-  while(NULL != pPass)
+  if (!_psSessionData->szHostHeader)
   {
-    switch(nState)
+    nBufLength = strlen(_psLogin->psServer->psHost->pHost) + 1 + log(params.nPort) + 1;
+    _psSessionData->szHostHeader = malloc(nBufLength + 1);
+    memset(_psSessionData->szHostHeader, 0, nBufLength + 1);
+    sprintf(_psSessionData->szHostHeader, "%s:%d", _psLogin->psServer->psHost->pHost, params.nPort);
+  }
+      
+  if (!_psSessionData->szUserAgent)
+  {
+    _psSessionData->szUserAgent = malloc(19);
+    memset(_psSessionData->szUserAgent, 0, 19);
+    sprintf(_psSessionData->szUserAgent, "Teh Forest Lobster");
+  }
+
+  while (nState != MSTATE_COMPLETE)
+  {
+    switch (nState)
     {
-    case HSTATE_NEW:
+    case MSTATE_NEW:
       // Already have an open socket - close it
       if (hSocket > 0)
         medusaDisconnect(hSocket);
 
-      if (login->psServer->psHost->iUseSSL > 0)
+      if (_psLogin->psServer->psHost->iUseSSL > 0)
         hSocket = medusaConnectSSL(&params);
       else
         hSocket = medusaConnect(&params);
 
       if (hSocket < 0)
       {
-        writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, login->psServer->pHostIP);
-        login->iResult = LOGIN_RESULT_UNKNOWN;
-        login->iStatus = LOGIN_DONE;
-        setPassResult(login, pPass);
+        writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, _psLogin->psServer->pHostIP);
+        _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
         return FAILURE;
       }
 
-      /* Set request parameters */
-      if (!_psSessionData->szDir) {
-        _psSessionData->szDir = malloc(1);
-        memset(_psSessionData->szDir, 0, 1);
-      }
-
-      if (!_psSessionData->szHostHeader) {
-        
-        nBufLength = strlen(login->psServer->psHost->pHost) + 1 + log(params.nPort) + 1;
-        _psSessionData->szHostHeader = malloc(nBufLength + 1);
-        memset(_psSessionData->szHostHeader, 0, nBufLength + 1);
-        sprintf(_psSessionData->szHostHeader, "%s:%d", login->psServer->psHost->pHost, params.nPort);
-      }
-      
-      if (!_psSessionData->szUserAgent) {
-        _psSessionData->szUserAgent = malloc(19);
-        memset(_psSessionData->szUserAgent, 0, 19);
-        sprintf(_psSessionData->szUserAgent, "Teh Forest Lobster");
-      }
-
       /* Get required authorization method */
       if (_psSessionData->nAuthType == AUTH_UNKNOWN)
       {
-        getAuthType(hSocket, _psSessionData, &login, params.nPort);
+        getAuthType(hSocket, _psSessionData, &_psLogin, params.nPort);
         
         if (_psSessionData->nAuthType == AUTH_UNKNOWN)
         {
-          login->iResult = LOGIN_RESULT_UNKNOWN;
-          login->iStatus = LOGIN_DONE;
+          _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           return FAILURE;
         }
         
-        nState = HSTATE_NEW;
+        nState = MSTATE_NEW;
       }
       else
-      nState = HSTATE_RUNNING;
+        nState = MSTATE_RUNNING;
       
       break;
-    case HSTATE_RUNNING:
-      nState = tryLogin(hSocket, _psSessionData, &login, params.nPort, user->pUser, pPass);
-      medusaDisconnect(hSocket);
-      hSocket = -1;
-      pPass = getNextPass(login->psServer->psAudit, user);
-      if (pPass == NULL)
+    case MSTATE_RUNNING:
+      nState = tryLogin(hSocket, _psSessionData, &_psLogin, params.nPort, psCredSet->psUser->pUser, psCredSet->pPass);
+
+      if (_psLogin->iResult != LOGIN_RESULT_UNKNOWN)
       {
-        writeError(ERR_DEBUG_MODULE, "Done with passwords, exiting http");
-        nState = HSTATE_EXITING;
-      }
-      else
-      {
-        nState = HSTATE_NEW;
+        if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
+        {
+          writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+          nState = MSTATE_EXITING;
+        }
+        else
+        {
+          if (psCredSet->iStatus == CREDENTIAL_DONE)
+          {
+            writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+          {
+            writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+            nState = MSTATE_NEW;
+          }
+          else
+            writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+        }
       }
+
       break;
-    case HSTATE_EXITING:
+    case MSTATE_EXITING:
       if (hSocket > 0)
         medusaDisconnect(hSocket);
       hSocket = -1;
+      nState = MSTATE_COMPLETE;
+      break;
     default:
       writeError(ERR_CRITICAL, "Unknown HTTP module state (%d). Exiting...", nState);
-      login->iResult = LOGIN_RESULT_UNKNOWN;
-      login->iStatus = LOGIN_DONE;
-      setPassResult(login, pPass);
+      _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+      break;
     }
   }
 
-  login->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -329,7 +367,6 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
 
 int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort)
 {
-  int iRet;
   unsigned char* bufSend = NULL;
   unsigned char* bufReceive = NULL;
   int nReceiveBufferSize = 0;
@@ -394,15 +431,28 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int n
 
 int sendAuthBasic(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
-  unsigned char* bufSend;
-  unsigned char* bufReceive;
-  unsigned char* szEncodedAuth;
+  unsigned char* bufSend = NULL;
+  unsigned char* szEncodedAuth = NULL;
+  unsigned char* szLoginDomain = NULL;
   int nSendBufferSize = 0;
   int nRet = SUCCESS;
 
-  writeError(ERR_DEBUG_MODULE, "[%s] Base64 encoding: %s:%s", MODULE_NAME, szLogin, szPassword);
-  szEncodedAuth = basic_authentication_encode(szLogin, szPassword);
+  if (_psSessionData->szDomain)
+  {
+    /* DOMAIN\USERNAME */
+    szLoginDomain = malloc(strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
+    memset(szLoginDomain, 0, strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
+    sprintf(szLoginDomain, "%s\\%s", _psSessionData->szDomain, szLogin);
+  }
+  else
+    szLoginDomain = szLogin;
+
+  writeError(ERR_DEBUG_MODULE, "[%s] Base64 encoding: %s:%s", MODULE_NAME, szLoginDomain, szPassword);
+  szEncodedAuth = basic_authentication_encode(szLoginDomain, szPassword);
   writeError(ERR_DEBUG_MODULE, "[%s] Base64 encoded data is: %s", MODULE_NAME, szEncodedAuth);
+
+  if (_psSessionData->szDomain)
+    FREE(szLoginDomain);
   
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 23 + strlen(szEncodedAuth) + 4;
@@ -426,29 +476,24 @@ int sendAuthBasic(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
 int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
-  unsigned char* bufSend;
-  unsigned char* bufReceive;
+  unsigned char* bufSend = NULL;
+  unsigned char* bufReceive = NULL;
   int nReceiveBufferSize = 0;
   int nSendBufferSize = 0;
-  unsigned char *szTmpBuf;
-  unsigned char *szTmpBuf64;
-  unsigned char *szTmpChall;
+  tSmbNtlmAuthRequest   sTmpReq;
+  tSmbNtlmAuthChallenge sTmpChall;
+  tSmbNtlmAuthResponse  sTmpResp;
+  unsigned char *szTmpBuf = NULL;
+  unsigned char *szTmpBuf64 = NULL;
 
   /* --- Send Type-1 NTLM request --- */
 
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthRequest) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthRequest));  
-  szTmpBuf64 = malloc(2 * sizeof(tSmbNtlmAuthRequest) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthRequest) + 2);
+  buildAuthRequest(&sTmpReq, 0, NULL, NULL);
 
-  buildAuthRequest((tSmbNtlmAuthRequest*)szTmpBuf, 0x00000202, NULL, NULL);
-  if (szTmpBuf == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] Failed to create initial NTLM request.", MODULE_NAME);
-    return FAILURE;
-  }
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpReq) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpReq) + 2);
 
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthRequest), szTmpBuf64);
+  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
   
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
@@ -466,11 +511,10 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
   
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
   FREE(bufSend);
 
-  /* Retrieve NTLM challenge from server */
+  /* --- Retrieve NTLM challenge from server --- */
   bufReceive = medusaReceiveLine(hSocket, &nReceiveBufferSize);
   if (bufReceive == NULL)
   {
@@ -496,30 +540,19 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   szTmpBuf = ((char*)index(szTmpBuf64, '\r'));
   szTmpBuf[0] = '\0';
 
-  szTmpChall = malloc( sizeof(tSmbNtlmAuthChallenge) );
-  memset(szTmpChall, 0, sizeof(tSmbNtlmAuthChallenge));
-
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64 + 23);
-  base64_decode(szTmpBuf64 + 23, szTmpChall);
+  base64_decode(szTmpBuf64 + 23, (char *)&sTmpChall);
   
   FREE(bufReceive);
- 
+
   /* --- Send Type-3 NTLM reply --- */
 
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthResponse) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthResponse));
+  buildAuthResponse(&sTmpChall, &sTmpResp, 0, szLogin, szPassword, _psSessionData->szDomain, NULL);
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthResponse) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthResponse) + 2);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpResp) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpResp) + 2);
 
-  buildAuthResponse((tSmbNtlmAuthChallenge*)szTmpChall, (tSmbNtlmAuthResponse*)szTmpBuf, 0, szLogin, szPassword, NULL, NULL);
-  if (szTmpBuf == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] Failed to create NTLM response.", MODULE_NAME);
-    return FAILURE;
-  }
-  
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
   
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
@@ -537,7 +570,6 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
   FREE(bufSend);
 
@@ -547,8 +579,8 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 /* http://www.ietf.org/rfc/rfc2617.txt */
 int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
-  unsigned char* bufSend;
-  unsigned char* bufReceive;
+  unsigned char* bufSend = NULL;
+  unsigned char* bufReceive = NULL;
   int nReceiveBufferSize = 0;
   int nSendBufferSize = 0;
   int i;
@@ -822,9 +854,10 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
 
 int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort, char* szLogin, char* szPassword)
 {
-  char* pReceiveBuffer;
-  int nReceiveBufferSize, nRet;
-  char* pTemp;
+  char* pReceiveBuffer = NULL;
+  int nReceiveBufferSize = 0;
+  int nRet = SUCCESS;
+  char* pTemp = NULL;
 
   switch(_psSessionData->nAuthType)
   {
@@ -832,7 +865,7 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
       writeError(ERR_DEBUG_MODULE, "[%s] No authentication required.", MODULE_NAME);
       (*login)->iResult = LOGIN_RESULT_SUCCESS;
       setPassResult(*login, szPassword);
-      return HSTATE_RUNNING;  
+      return MSTATE_NEW;  
       break;
     case AUTH_BASIC:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending Basic Authentication.", MODULE_NAME);
@@ -855,7 +888,7 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
     writeError(ERR_ERROR, "[%s] Failed during sending of authentication data.", MODULE_NAME);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;  
+    return MSTATE_EXITING;  
   }
 
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
@@ -865,10 +898,10 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
     writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;  
+    return MSTATE_EXITING;  
   }
     
-  while (strstr(pReceiveBuffer, "HTTP/1.") == NULL && (pReceiveBuffer[0] != '\0'))
+  while ((strstr(pReceiveBuffer, "HTTP/1.") == NULL) && (pReceiveBuffer[0] != '\0'))
   {
     FREE(pReceiveBuffer);
     pReceiveBuffer = medusaReceiveLine(hSocket, &nReceiveBufferSize);
@@ -879,15 +912,17 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
     writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;  
+    return MSTATE_EXITING;  
   }
 
-  pTemp = ((char*)index(pReceiveBuffer, ' ')) + 1;
+  pTemp = strstr(pReceiveBuffer, "HTTP/1.");
+  pTemp = ((char*)index(pTemp, ' ')) + 1;
   if (*pTemp == '2' || strncmp(pTemp, "301", 3) == 0 ) 
   {
     (*login)->iResult = LOGIN_RESULT_SUCCESS;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;
+    FREE(pReceiveBuffer);
+    return MSTATE_NEW;
   } 
   else 
   {
@@ -896,21 +931,19 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
       writeVerbose(VB_GENERAL, "Unusual return code for %s:%s (%.3s)", szLogin, szPassword, pTemp);
       (*login)->iResult = LOGIN_RESULT_UNKNOWN;
       setPassResult(*login, szPassword);
-      return HSTATE_RUNNING;   
+      FREE(pReceiveBuffer);
+      return MSTATE_EXITING;   
     }
     
     if (strncmp(pTemp, "404", 3) == 0)
-    {
       writeVerbose(VB_GENERAL, "Page not found at %s:%d", (*login)->psServer->pHostIP, nPort);
-      (*login)->iResult = LOGIN_RESULT_FAIL;
-      setPassResult(*login, szPassword);
-      return HSTATE_RUNNING;   
-    }
     
-    writeError(ERR_DEBUG, "Login failed: error code was %.3s", pTemp);
+    writeError(ERR_DEBUG_MODULE, "Login failed: error code was %.3s", pTemp);
     (*login)->iResult = LOGIN_RESULT_FAIL;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;   
+    FREE(pReceiveBuffer);
+
+    return MSTATE_NEW;
   }
 }
 
@@ -939,7 +972,6 @@ void showUsage()
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
   writeVerbose(VB_NONE, "** Module was not properly built. Is OPENSSL installed correctly? **");
   writeVerbose(VB_NONE, "");
-  return FAILURE;
 }
 
 int go(sLogin* logins, int argc, char *argv[])
