@@ -41,21 +41,18 @@
 #define MODULE_NAME    "imap.mod"
 #define MODULE_AUTHOR  "pMonkey <pmonkey@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for IMAP sessions"
-#define MODULE_VERSION    "1.2.0"
-#define MODULE_VERSION_SVN "$Id: imap.c 987 2008-10-10 17:43:41Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: imap.c 1259 2009-11-16 04:19:46Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define BUF_SIZE 300
 
 #define PORT_IMAP  143
 #define PORT_IMAPS 993
 
+#define RECEIVE_DELAY_1 20 * 1000000
+#define RECEIVE_DELAY_2 0.5 * 1000000
+
 #define AUTH_UNKNOWN 0
 #define AUTH_LOGIN 1
 #define AUTH_PLAIN 2
@@ -64,6 +61,7 @@
 typedef struct __MODULE_DATA {
   char *szTag;
   int nAuthType;
+  char* szDomain;
 } _MODULE_DATA;
 
 // Tells us whether we are to continue processing or not
@@ -71,7 +69,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -111,8 +110,25 @@ void showUsage()
   writeVerbose(VB_NONE, "Available module options:");
   writeVerbose(VB_NONE, "  TAG:? (Default: gerg)");
   writeVerbose(VB_NONE, "  AUTH:? (Authentication Type (LOGIN/PLAIN/NTLM). Default: automatic)");
+  writeVerbose(VB_NONE, "  DOMAIN:? [optional]");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "Usage example: \"-M imap -m TAG:A0001 -m AUTH:PLAIN");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "The DOMAIN option should supply the specified domain appropriately,");
+  writeVerbose(VB_NONE, "regardless of authentication type. The domain can also be supplied ");
+  writeVerbose(VB_NONE, "via the username field, but the format appears to differ by auth type.");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "Example 1: NTLM authentication with DOMAIN option");
+  writeVerbose(VB_NONE, "   \"medusa -M imap -m AUTH:NTLM -m DOMAIN:FOODOM -h host -u foo -p bar\"");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "Example 2: NTLM authentication with domain via username");
+  writeVerbose(VB_NONE, "   \"medusa -M imap -m AUTH:NTLM -h host -u foo@domain -p bar\"");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "* If no domain is specified when using NTLM authentication, the server");
+  writeVerbose(VB_NONE, "supplied value will be used.");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "Example 3: LOGIN authentication with domain via username");
+  writeVerbose(VB_NONE, "   \"medusa -M imap -m AUTH:LOGIN -h host -u 'domain\\\\foo' -p bar\"");
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
@@ -133,7 +149,7 @@ int go(sLogin* logins, int argc, char *argv[])
   else 
   {
     // Parameters are good - make module go now
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
 
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
@@ -171,6 +187,20 @@ int go(sLogin* logins, int argc, char *argv[])
         else
           writeError(ERR_WARNING, "Invalid value for method AUTH.");
       }
+      else if (strcmp(pOpt, "DOMAIN") == 0)
+      {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if ( pOpt )
+        {
+          psSessionData->szDomain = malloc(strlen(pOpt) + 1);
+          memset(psSessionData->szDomain, 0, strlen(pOpt) + 1); 
+          strncpy((char *) psSessionData->szDomain, pOpt, strlen(pOpt));
+        }
+        else
+          writeError(ERR_WARNING, "Method DOMAIN requires value to be set.");
+      }
       else
       {
         writeError(ERR_WARNING, "Invalid method: %s.", pOpt);
@@ -182,6 +212,7 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(psSessionData, logins);
   }  
 
+  FREE(psSessionData);
   return SUCCESS;
 }
 
@@ -189,14 +220,28 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
   unsigned char bufSend[BUF_SIZE];
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
@@ -214,24 +259,9 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
     sprintf(_psSessionData->szTag, "gerg");
   }
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         // Already have an open socket - close it
@@ -247,15 +277,12 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
         if (initConnection(_psSessionData, hSocket, &params) == FAILURE)
         {
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          FREE(_psSessionData);
           return FAILURE;
         }
 
@@ -264,15 +291,53 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, _psSessionData, &psLogin, user->pUser, pPass);
-        if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        /* The IMAP service may be configured to drop connections after an arbitrary number of failed
+           logon attempts. We will reuse the established connection to send authentication attempts 
+           until that disconnect happens. At that point the connection should be reestablished. */
+        if ( medusaCheckSocket(hSocket) )
+        {
+          nState = tryLogin(hSocket, _psSessionData, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
+
+          if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
+          {
+            if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+            {
+              writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else
+            {
+              if (psCredSet->iStatus == CREDENTIAL_DONE)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+                nState = MSTATE_EXITING;
+              }
+              else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+                nState = MSTATE_NEW;
+              }
+              else
+                writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+            }
+          }
+        }
+        else
+        {
+          writeError(ERR_NOTICE, "[%s] Socket is no longer valid. Server likely dropped connection. Establishing new session.", MODULE_NAME);
+          nState = MSTATE_NEW;
+
+          if (hSocket > 0)
+            medusaDisconnect(hSocket);
+          hSocket = -1;
+        }
+
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -280,12 +345,11 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
- 
-  psLogin->iStatus = LOGIN_DONE;
+
+  FREE(psCredSet); 
   return SUCCESS;
 }
 
@@ -298,8 +362,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   int nSendBufferSize = 0;
 
   /* Retrieve IMAP server banner */
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\* OK .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Failed to retrieve IMAP server banner. Exiting...", MODULE_NAME);
     return FAILURE; 
@@ -338,15 +401,14 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   FREE(bufSend);
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK CAPABILITY .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: No OK message received for CAPABILITY request.", MODULE_NAME);
     return FAILURE;
   }
 
-  /* If server supports STARTTLS, we use it */
-  if (strstr(bufReceive, "STARTTLS") != NULL)
+  /* If server supports STARTTLS and we are not already within a SSL connection, let's use it. */
+  if ((params->nUseSSL == 0) && (strstr(bufReceive, "STARTTLS") != NULL))
   {
     FREE(bufReceive);
 
@@ -362,22 +424,22 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       return FAILURE;
     }
     FREE(bufSend);
-
+  
     nReceiveBufferSize = 0;
-    bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-    if (bufReceive == NULL)
+    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, " OK .*\r\n") == FAILURE) || (bufReceive == NULL))
     {
-      writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
+      writeError(ERR_ERROR, "[%s] Failed: No OK message received for STARTTLS request.", MODULE_NAME);
       return FAILURE;
     }
     /* OK Begin TLS negotiation now. */
-    else if (strstr(bufReceive, " OK ") != NULL)
+    else
     {
       FREE(bufReceive);
 
+      params->nSSLVersion = 3.1; /* Force the use of TLSv1 */
       if (medusaConnectSocketSSL(params, hSocket) < 0)
       {
-        writeError(ERR_ERROR, "[%s] Failed to establish SSLv3 connection.", MODULE_NAME);
+        writeError(ERR_ERROR, "[%s] Failed to establish TLSv1 connection.", MODULE_NAME);
         return FAILURE;
       }
 
@@ -397,10 +459,9 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       FREE(bufSend);
 
       nReceiveBufferSize = 0;
-      bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-      if (bufReceive == NULL)
+      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK CAPABILITY .*\r\n") == FAILURE) || (bufReceive == NULL))
       {
-        writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
+        writeError(ERR_ERROR, "[%s] Failed: No OK message received for CAPABILITY request.", MODULE_NAME);
         return FAILURE;
       }
     }
@@ -448,8 +509,16 @@ int sendAuthLogin(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
-  writeError(ERR_DEBUG_MODULE, "[%s] Sending authenticate login value: %s %s", MODULE_NAME, szLogin, szPassword); 
-  sprintf(bufSend, "%s LOGIN \"%s\" \"%s\"\r\n", _psSessionData->szTag, szLogin, szPassword);
+  if (_psSessionData->szDomain) 
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] Sending authenticate login value: %s\\\\%s %s", MODULE_NAME, _psSessionData->szDomain, szLogin, szPassword); 
+    sprintf(bufSend, "%s LOGIN \"%s\\\\%s\" \"%s\"\r\n", _psSessionData->szTag, _psSessionData->szDomain, szLogin, szPassword);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] Sending authenticate login value: %s %s", MODULE_NAME, szLogin, szPassword); 
+    sprintf(bufSend, "%s LOGIN \"%s\" \"%s\"\r\n", _psSessionData->szTag, szLogin, szPassword);
+  }
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
@@ -484,13 +553,7 @@ int sendAuthPlain(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   FREE(bufSend); 
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strstr(bufReceive,"+") == NULL) 
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ *\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] IMAP server did not respond with \"+\" to AUTHENTICATE PLAIN request.", MODULE_NAME);
     writeError(ERR_ERROR, "[%s] IMAP server sent the following response: %s", MODULE_NAME, bufReceive);
@@ -542,11 +605,13 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 {
   unsigned char* bufSend = NULL;
   unsigned char* bufReceive = NULL;
-  unsigned char* szTmpBuf = NULL;
-  unsigned char* szTmpBuf64 = NULL;
-  unsigned char* szTmpChall = NULL;
   int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
+  tSmbNtlmAuthRequest   sTmpReq;
+  tSmbNtlmAuthChallenge sTmpChall;
+  tSmbNtlmAuthResponse  sTmpResp;
+  unsigned char* szTmpBuf = NULL;
+  unsigned char* szTmpBuf64 = NULL;
 
   /* --- Send initial AUTHENTICATE NTLM command --- */
   nSendBufferSize = strlen(_psSessionData->szTag) + 21;
@@ -562,35 +627,28 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 
   /* Server should respond with an empty challenge, consisting simply of a "+" */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strstr(bufReceive, "+") == NULL) 
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] IMAP server did not respond with \"+\" to AUTHENTICATE NTLM request.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] IMAP server sent the following response: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
   FREE(bufReceive);
 
   /* --- Send Base-64 encoded Type-1 message --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthRequest) );  
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthRequest));
+  buildAuthRequest(&sTmpReq, 0, NULL, NULL);
+  
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpReq) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpReq) + 2);
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthRequest) + 2);  
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthRequest) + 2);
-
-  buildAuthRequest((tSmbNtlmAuthRequest*)szTmpBuf, 0, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
   sprintf(bufSend, "%s\r\n", szTmpBuf64);
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
@@ -603,13 +661,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   /* Server should respond with a Base-64 encoded Type-2 challenge message. The challenge response format is 
      specified by RFC 1730 ("+", followed by a space, followed by the challenge message). */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strncmp(bufReceive, "+ ", 2) != 0)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Server did not send valid Type-2 challenge response.", MODULE_NAME);
     return FAILURE;
@@ -618,22 +670,20 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   szTmpBuf = ((char*)index(bufReceive, '\r'));
   szTmpBuf[0] = '\0';
 
-  szTmpChall = malloc( sizeof(tSmbNtlmAuthChallenge) );  
-  memset(szTmpChall, 0, sizeof(tSmbNtlmAuthChallenge));
-  
-  base64_decode(bufReceive + 2, szTmpChall);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, bufReceive + 2);
+  base64_decode(bufReceive + 2, (char *)&sTmpChall);
 
   FREE(bufReceive);
   
   /* --- Calculate and send Base-64 encoded Type 3 response --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthResponse) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthResponse));
-  
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthResponse) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthResponse) + 2);
+ 
+  buildAuthResponse(&sTmpChall, &sTmpResp, 0, szLogin, szPassword, _psSessionData->szDomain, NULL); 
 
-  buildAuthResponse((tSmbNtlmAuthChallenge*)szTmpChall, (tSmbNtlmAuthResponse*)szTmpBuf, 0, szLogin, szPassword, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpResp) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpResp) + 2);
+
+  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
@@ -646,9 +696,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
-  FREE(szTmpChall);
   FREE(bufSend);
 
   /* Server should validate the response and indicate the result of authentication.
@@ -700,10 +748,9 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** psLogin, char*
  
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, ".*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_ERROR;
     nRet = MSTATE_EXITING;
   }
