@@ -2,7 +2,7 @@
 **   MySQL Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -40,23 +40,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include "module.h"
 
 #define MODULE_NAME    "mysql.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for MySQL sessions"
-#define MODULE_VERSION    "1.2"
-#define MODULE_VERSION_SVN "$Id: mysql.c 1052 2009-01-21 22:27:47Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: mysql.c 1225 2009-10-13 18:55:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define BUF_SIZE 300
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_MYSQL 3306
 #define PROTO_UNKNOWN 0
 #define PROTO_OLD 1
@@ -69,13 +62,13 @@ typedef struct __MYSQL_DATA {
   int hashFlag;
 } _MYSQL_DATA;
 
-
 // Tells us whether we are to continue processing or not
 enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -176,20 +169,35 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MYSQL_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
@@ -197,24 +205,9 @@ int initModule(sLogin* psLogin, _MYSQL_DATA *_psSessionData)
     params.nPort = PORT_MYSQL;
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         // Already have an open socket - close it
@@ -230,7 +223,6 @@ int initModule(sLogin* psLogin, _MYSQL_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
@@ -239,15 +231,37 @@ int initModule(sLogin* psLogin, _MYSQL_DATA *_psSessionData)
 
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -255,12 +269,11 @@ int initModule(sLogin* psLogin, _MYSQL_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  psLogin->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -684,7 +697,6 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
   {
     writeError(ERR_ERROR, "[%s] Failed to initialize MySQL connection (%s).", MODULE_NAME, (*psLogin)->psServer->pHostIP);
     (*psLogin)->iResult = LOGIN_RESULT_ERROR;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     return MSTATE_EXITING;
   }
 
@@ -692,7 +704,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
   if (strlen(szSessionSalt) == 8 || _psSessionData->protoFlag == PROTO_OLD)
   {
     if (_psSessionData->protoFlag == PROTO_OLD) {
-      writeError(ERR_DEBUG, "[%s] Using older style authentication based on previous server response.", MODULE_NAME);
+      writeError(ERR_DEBUG_MODULE, "[%s] Using older style authentication based on previous server response.", MODULE_NAME);
     }
     
     iRet = MySQLPrepareAuthOld(_psSessionData, szLogin, szPassword, szSessionSalt, &szResponse, &iResponseLength);
@@ -743,7 +755,6 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
     {
       writeError(ERR_ERROR, "[%s] failed: MYSQL VERSION IS NEWER\n", MODULE_NAME);
       (*psLogin)->iResult = LOGIN_RESULT_ERROR;
-      (*psLogin)->iStatus = LOGIN_FAILED;
       iReturnCode = MSTATE_EXITING;
     }
     else 
@@ -757,7 +768,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
        the server is nice enough to tell us and allow us to reauthenticate.
     */
 
-    writeError(ERR_DEBUG, "[%s] Server requested older authentication type. It is likely the remote account exists and has an older style password hash.", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "[%s] Server requested older authentication type. It is likely the remote account exists and has an older style password hash.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
 
     /* Attempt authentication again using old-style password hash and existing connection */
@@ -801,7 +812,6 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
       if (bufReceive[5] == 0xe3 && bufReceive[6] == 0x04) {
         writeError(ERR_ERROR, "%s failed: MYSQL VERSION IS NEWER\n", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
-        (*psLogin)->iStatus = LOGIN_FAILED;
         iReturnCode = MSTATE_EXITING;
       }
       else
@@ -813,7 +823,6 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
   {
     writeError(ERR_ERROR, "%s: Unknown response code received from server: %X", MODULE_NAME, bufReceive[4]);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     iReturnCode = MSTATE_EXITING;
   }
 
