@@ -2,7 +2,7 @@
 **   NCP Password/HASH Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 #define MODULE_NAME    "ncp.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for NCP sessions"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: ncp.c 766 2007-05-31 21:37:34Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: ncp.c 1229 2009-10-14 22:21:37Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBNCP_WARNING "No usable LIBNCP. Module disabled."
@@ -60,7 +60,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -138,7 +139,7 @@ int go(sLogin* logins, int argc, char *argv[])
   } 
   else 
   {
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
  
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
@@ -164,110 +165,138 @@ int go(sLogin* logins, int argc, char *argv[])
       else
          writeError(ERR_WARNING, "Invalid method: %s.", pOpt);
       
-      free(pOptTmp);
+      FREE(pOptTmp);
     }
  
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _NCP_DATA *_psSessionData)
 {
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass = NULL;
   char *szUserContext = NULL;
-  sUser* user = psLogin->psUser;
   long NCPErrorCode;
+  sCredentialSet *psCredSet = NULL;
+  int i = 0;
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
 
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
   {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
   }
-
-  if (_psSessionData->context != NULL)
+  else if (psCredSet->psUser)
   {
-    szUserContext = malloc(strlen(user->pUser) + strlen(_psSessionData->context) + 1);
-    memset(szUserContext, 0, strlen(user->pUser) + strlen(_psSessionData->context) + 1);
-    strncpy(szUserContext, user->pUser, strlen(user->pUser));
-    strncpy(szUserContext + strlen(user->pUser), _psSessionData->context, strlen(_psSessionData->context));
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
   }
   else
-    szUserContext = user->pUser;
-  writeError(ERR_DEBUG, "[%s] Set user context: %s", MODULE_NAME, szUserContext);
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
 
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
+        FREE(szUserContext);
+        if (_psSessionData->context != NULL)
+        {
+          szUserContext = malloc(strlen(psCredSet->psUser->pUser) + strlen(_psSessionData->context) + 1);
+          memset(szUserContext, 0, strlen(psCredSet->psUser->pUser) + strlen(_psSessionData->context) + 1);
+          strncpy(szUserContext, psCredSet->psUser->pUser, strlen(psCredSet->psUser->pUser));
+          strncpy(szUserContext + strlen(psCredSet->psUser->pUser), _psSessionData->context, strlen(_psSessionData->context));
+        }
+        else
+          szUserContext = psCredSet->psUser->pUser;
+        
+        writeError(ERR_DEBUG_MODULE, "[%s] Set user context: %s", MODULE_NAME, szUserContext);
+
         NCPErrorCode = ncp_find_conn_spec3(psLogin->psServer->pHostIP, szUserContext, "", 1, 1 ? ~0U : getuid(), 0, &_psSessionData->spec);
         if (NCPErrorCode)
         {
           writeError(ERR_ERROR, "[%s] Failed to find an appropriate connection: %d.", MODULE_NAME, NCPErrorCode);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          free(_psSessionData);
           return FAILURE;
         }
 
-        NCPErrorCode = NWCCOpenConnByName(NULL, _psSessionData->spec.server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_NEW_CONN, NWCC_RESERVED, &_psSessionData->conn);
-        if (NCPErrorCode)
+        /* Initiate NCP session connection - retry if necessary */ 
+        writeError(ERR_DEBUG_MODULE, "Attempting to establish connection with NCP server.");
+        for (i = 1; i <= psLogin->psServer->psHost->iRetries + 1; i++) 
         {
-          writeError(ERR_ERROR, "[%s] Failed while attempting to find server: %d.", MODULE_NAME, NCPErrorCode);
-          psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          free(_psSessionData);
-          return FAILURE;
+
+          NCPErrorCode = NWCCOpenConnByName(NULL, _psSessionData->spec.server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_NEW_CONN, NWCC_RESERVED, &_psSessionData->conn);
+          if (NCPErrorCode)
+          {
+            writeError(ERR_ERROR, "[%s] Failed establishing NCP session (%d/%d): Error Code: %d Host: %s User: %s Pass: %s", MODULE_NAME, i, psLogin->psServer->psHost->iRetries + 1, NCPErrorCode, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
+
+            if (i == psLogin->psServer->psHost->iRetries + 1) {
+              psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+              return FAILURE;
+            }
+          }
         }
 
         writeError(ERR_DEBUG_MODULE, "Connected");
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(0, &psLogin, _psSessionData, user->pUser, pPass);
+        nState = tryLogin(0, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         ncp_close(_psSessionData->conn);
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module (%d) state %d host: %s", MODULE_NAME, psLogin->iId, nState, psLogin->psServer->pHostIP);
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
-        free(_psSessionData);
         return FAILURE;
     }  
   }
  
-  psLogin->iStatus = LOGIN_DONE;
   if (_psSessionData->context != NULL)
-    if (szUserContext != NULL) free(szUserContext);
-  free(_psSessionData);
+    if (szUserContext != NULL) FREE(szUserContext);
+  
+  FREE(psCredSet);
   return SUCCESS;
 }
 
 int tryLogin(int hSocket, sLogin** psLogin, _NCP_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
   int i, iRet;
-  short int NCPErrorCode;
-  char *pErrorMsg;
+  short int NCPErrorCode = 0;
+  char *pErrorMsg = NULL;
   char ErrorCode[12];
   int object_type = NCP_BINDERY_USER;
 
@@ -364,12 +393,12 @@ int tryLogin(int hSocket, sLogin** psLogin, _NCP_DATA* _psSessionData, char* szL
       break;
     case 0x89F0:  /* Incorrect password - BIND Authenticator */
     case 0xFD63:  /* Incorrect password - NDS Authenticator */
-      writeError(ERR_DEBUG, "[%s] Incorrect password. Error code: %X", MODULE_NAME, NCPErrorCode);
+      writeError(ERR_DEBUG_MODULE, "[%s] Incorrect password. Error code: %X", MODULE_NAME, NCPErrorCode);
       (*psLogin)->iResult = LOGIN_RESULT_FAIL;
       iRet = MSTATE_RUNNING;
       break;
     default:
-      writeError(ERR_DEBUG, "[%s] Failed to open connection. Error code: %X", MODULE_NAME, NCPErrorCode);
+      writeError(ERR_DEBUG_MODULE, "[%s] Failed to open connection. Error code: %X", MODULE_NAME, NCPErrorCode);
       sprintf(ErrorCode, "0x%8.8X:", NCPErrorCode);
       (*psLogin)->pErrorMsg = malloc( strlen(ErrorCode) + strlen(pErrorMsg) + 1);
       memset((*psLogin)->pErrorMsg, 0, strlen(ErrorCode) + strlen(pErrorMsg) + 1);
@@ -410,7 +439,6 @@ void showUsage()
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
   writeVerbose(VB_NONE, "** Module was not properly built. Are the ncpfs headers and static library installed correctly? **");
   writeVerbose(VB_NONE, "");
-  return FAILURE;
 }
 
 int go(sLogin* logins, int argc, char *argv[])
