@@ -5,7 +5,7 @@
 **    http://www.mibsoftware.com/userkt/nntpext/0032.htm
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2007 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -37,18 +37,12 @@
 #define MODULE_NAME    "nntp.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for NNTP sessions"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: nntp.c 976 2008-09-26 20:15:55Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: nntp.c 1225 2009-10-13 18:55:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300 
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_NNTP  119
 #define PORT_NNTPS 563
 
@@ -61,7 +55,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -93,7 +88,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -124,20 +118,35 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
@@ -147,22 +156,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
     params.nPort = PORT_NNTP;
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
     switch(nState)
     {
@@ -180,7 +174,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "[%s] failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
         
@@ -188,12 +181,37 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
  
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
-        pPass = NULL;
+        if (hSocket > 0)
+          terminateConnection(hSocket, &psLogin);
+        hSocket = -1;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -201,15 +219,11 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  if (hSocket > 0)
-    terminateConnection(hSocket, &psLogin);
- 
-  psLogin->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -244,7 +258,6 @@ int initConnection(int hSocket, sLogin** psLogin)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data. Exiting...", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     return MSTATE_EXITING;
   }
   else if ((strstr(bufReceive,"200") != NULL)) 
@@ -256,14 +269,12 @@ int initConnection(int hSocket, sLogin** psLogin)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Too many connections from host recieved by server.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     return MSTATE_EXITING;
   }
   else
   {
     writeError(ERR_ERROR, "[%s] Failed to retrieve NNTP service banner.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     return MSTATE_EXITING;
   }
      
@@ -275,7 +286,6 @@ int initConnection(int hSocket, sLogin** psLogin)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     return MSTATE_EXITING;
   }
  
@@ -285,7 +295,6 @@ int initConnection(int hSocket, sLogin** psLogin)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     iRet = MSTATE_EXITING;
   }
   /* 480 Authentication Required */ 
@@ -298,7 +307,6 @@ int initConnection(int hSocket, sLogin** psLogin)
   {
     writeError(ERR_ERROR, "[%s] Server does not appear to require authentication.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     iRet = MSTATE_EXITING;
   }
 
