@@ -2,7 +2,7 @@
 **   POP3 Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -35,18 +35,12 @@
 #define MODULE_NAME    "pop3.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for POP3 sessions"
-#define MODULE_VERSION    "1.2"
-#define MODULE_VERSION_SVN "$Id: pop3.c 991 2008-10-20 16:36:46Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: pop3.c 1259 2009-11-16 04:19:46Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300 
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_POP3  110
 #define PORT_POP3S 995
 
@@ -70,7 +64,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -101,7 +96,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -111,11 +105,13 @@ void showUsage()
   writeVerbose(VB_NONE, "Available module options:");
   writeVerbose(VB_NONE, "  MODE:? (NORMAL, AS400) [optional]");
   writeVerbose(VB_NONE, "    Sets the mode for error detection.");
-   writeVerbose(VB_NONE, " AUTH:? (Authentication Type (USER/PLAIN/LOGIN/NTLM). Default: automatic)");
+  writeVerbose(VB_NONE, " AUTH:? (Authentication Type (USER/PLAIN/LOGIN/NTLM). Default: automatic)");
   writeVerbose(VB_NONE, "    Module will query service for accepted methods via an \"AUTH\" request.");
-  writeVerbose(VB_NONE, "    USER (clear-text), SASL PLAIN,  SASL LOGIN, and SASL NTLM authentication methods are supported.");
+  writeVerbose(VB_NONE, "    USER (clear-text), SASL PLAIN, SASL LOGIN, and SASL NTLM authentication methods are supported.");
   writeVerbose(VB_NONE, "  DOMAIN:? [optional]");
-  writeVerbose(VB_NONE, "    Appends domain to username (e.g. user@domain.com).");
+  writeVerbose(VB_NONE, "    AUTH USER - Appends domain to username (e.g. user@domain.com).");
+  writeVerbose(VB_NONE, "    AUTH NTLM - Supplies specified domain during NTLM authentication. The default");
+  writeVerbose(VB_NONE, "                behaviour is to use the server supplied domain value.");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "  Usage example: ");
   writeVerbose(VB_NONE, "    \"medusa -M pop3 -m MODE:AS400 -U accounts.txt -p password\"");
@@ -184,7 +180,8 @@ int go(sLogin* logins, int argc, char *argv[])
 
         if ( pOpt )
         {
-          psSessionData->szDomain = malloc(strlen(pOpt));
+          psSessionData->szDomain = malloc(strlen(pOpt) + 1);
+          memset(psSessionData->szDomain, 0, strlen(pOpt) + 1);
           strncpy((char *) psSessionData->szDomain, pOpt, strlen(pOpt));
         }
         else
@@ -199,7 +196,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
@@ -207,13 +205,29 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
   unsigned char bufSend[BUF_SIZE];
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  unsigned char* bufReceive;
+  int nReceiveBufferSize = 0;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
@@ -223,24 +237,9 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
     params.nPort = PORT_POP3;
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         // Already have an open socket - close it
@@ -256,25 +255,15 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
         /* establish initial connection */
         nReceiveBufferSize = 0;
-        bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-        if (bufReceive == NULL)
+        if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n") == FAILURE) || (bufReceive == NULL))
         {
-          writeError(ERR_ERROR, "%s failed: medusaReceive returned no data. Exiting...", MODULE_NAME);
+          writeError(ERR_DEBUG_MODULE, "%s failed: Server did not respond with '+OK'. Exiting...", MODULE_NAME);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          nState = MSTATE_EXITING;
-        }
-        else if (bufReceive[0] != '+')
-        {
-          writeError(ERR_DEBUG_MODULE, "%s failed: Server did not respond with '+'. Exiting...", MODULE_NAME);
-          psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           nState = MSTATE_EXITING;
         }
         else
@@ -291,42 +280,46 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
            permitted in the current state. "CAPA" can be used to test for its
            presence. Are there cases where "STLS" may not be implemented?
         */
-        memset(bufSend, 0, BUF_SIZE);
-        sprintf(bufSend, "STLS\r\n");
-        if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
-        {
-          writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
-          return FAILURE;
-        }
 
-        nReceiveBufferSize = 0;
-        bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-        if (bufReceive == NULL)
+        /* Initiate STLS only if we don't already have a SSL connection */
+        if (psLogin->psServer->psHost->iUseSSL == 0)
         {
-          writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
-          return FAILURE;
-        }
-        /*
-          [SUPPORTED]     +OK Begin TLS negotiation / +OK Ready to start TLS
-          [NOT SUPPORTED] +OK STLS completed
-          [ERROR]         -ERR Command not permitted when TLS active
-        */
-        else if ((strstr(bufReceive, "+OK Begin TLS negotiation") != NULL) ||
-                 (strstr(bufReceive, "+OK Ready to start TLS") != NULL))
-        {
-          FREE(bufReceive);
-
-          writeError(ERR_DEBUG_MODULE, "[%s] Starting TLS negotiation.", MODULE_NAME);
-          if (medusaConnectSocketSSL(&params, hSocket) < 0)
+          memset(bufSend, 0, BUF_SIZE);
+          sprintf(bufSend, "STLS\r\n");
+          if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
           {
-            writeError(ERR_ERROR, "[%s] Failed to establish SSLv3 connection.", MODULE_NAME);
+            writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
             return FAILURE;
           }
-        }
-        else
-        {
-          writeError(ERR_DEBUG_MODULE, "[%s] TLS negotiation not available.", MODULE_NAME);
-          FREE(bufReceive);
+  
+          nReceiveBufferSize = 0;
+          if (medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n|-ERR.*\r\n") == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, bufReceive);
+            return FAILURE;
+          }
+          /*
+            [SUPPORTED]     +OK Begin TLS negotiation / +OK Ready to start TLS
+            [NOT SUPPORTED] +OK STLS completed
+            [ERROR]         -ERR Command not permitted when TLS active
+          */
+          else if (strstr(bufReceive, "+OK") != NULL)
+          {
+            FREE(bufReceive);
+  
+            writeError(ERR_DEBUG_MODULE, "[%s] Starting TLS negotiation.", MODULE_NAME);
+            params.nSSLVersion = 3.1; /* Force the use of TLSv1 */
+            if (medusaConnectSocketSSL(&params, hSocket) < 0)
+            {
+              writeError(ERR_ERROR, "[%s] Failed to establish SSLv3 connection.", MODULE_NAME);
+              return FAILURE;
+            }
+          }
+          else
+          {
+            writeError(ERR_DEBUG_MODULE, "[%s] TLS negotiation not available.", MODULE_NAME);
+            FREE(bufReceive);
+          }
         }
   
         /* Query service for accepted authentication methods */
@@ -337,7 +330,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           if (_psSessionData->nAuthType == AUTH_UNKNOWN)
           {
             psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-            psLogin->iStatus = LOGIN_FAILED;
             return FAILURE;
           }
         }
@@ -349,9 +341,31 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
            until that disconnect happens. At that point the connection should be reestablished. */
         if ( medusaCheckSocket(hSocket) )
         {
-          nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
+          nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
           if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-            pPass = getNextPass(psLogin->psServer->psAudit, user);
+          {
+            if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+            {
+              writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else
+            {
+              if (psCredSet->iStatus == CREDENTIAL_DONE)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+                nState = MSTATE_EXITING;
+              }
+              else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+                nState = MSTATE_NEW;
+              }
+              else
+                writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+            }
+          }
         }
         else
         {
@@ -362,13 +376,12 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
             medusaDisconnect(hSocket);
           hSocket = -1;
         }
-
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -376,14 +389,11 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  if (psLogin->iStatus != LOGIN_FAILED)
-    psLogin->iStatus = LOGIN_DONE;
-  
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -391,7 +401,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 
 int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
 {
-  char* bufReceive;
+  unsigned char* bufReceive;
   unsigned char* bufSend;
   int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
@@ -407,10 +417,9 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
   }
   FREE(bufSend);
 
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK .*\r\n\\.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data. Exiting...", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Server did not respond that it supported any of the authentication types we handle (USER, LOGIN, and NTLM). Use the AUTH module option to force the use of an authentication type: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
   else if ((strstr(bufReceive,"USER") != NULL))
@@ -441,11 +450,6 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
       return FAILURE;
     }
   }
-  else
-  {
-    writeError(ERR_ERROR, "[%s] Failed: Server did not respond that it supported any of the authentication types we handle (USER, LOGIN, and NTLM). Use the AUTH module option to force the use of an authentication type.", MODULE_NAME);
-    return FAILURE;
-  }
 
   return SUCCESS;
 }
@@ -473,10 +477,9 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   }
  
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Server did not respond as expected to USER authentication attempt: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
   else if (strstr(bufReceive, " signing off."))
@@ -495,11 +498,6 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     writeError(ERR_ERROR, "[%s] Server does not accept clear-text password authentication.", MODULE_NAME);
     return FAILURE;
   }
-  else if (bufReceive[0] != '+')
-  {
-    writeError(ERR_ERROR, "[%s] Failed: Server did not respond as expected to USER authentication attempt.", MODULE_NAME);
-    return FAILURE;
-  }
  
   /* send password */
   memset(bufSend, 0, sizeof(bufSend));
@@ -595,17 +593,16 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded username prompt */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strstr(bufReceive, "+ ") == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ .*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] POP3 server did not respond with \"+ \" to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
   }
+  else if (strstr(bufReceive,"-ERR The specified authentication package is not supported.") != NULL) 
+  {
+    writeError(ERR_ERROR, "[%s] Server response: The specified authentication package is not supported.", MODULE_NAME);
+    return FAILURE;
+  }
 
   szTmpBuf = ((char*)index(bufReceive, '\r'));
   szTmpBuf[0] = '\0';
@@ -633,13 +630,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded password prompt */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strstr(bufReceive, "+ ") == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] POP3 server did not respond with \"+ \" to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
@@ -682,11 +673,13 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 {
   unsigned char* bufSend = NULL;
   unsigned char* bufReceive = NULL;
+  int nReceiveBufferSize = 0;
+  int nSendBufferSize = 0;
+  tSmbNtlmAuthRequest   sTmpReq;
+  tSmbNtlmAuthChallenge sTmpChall;
+  tSmbNtlmAuthResponse  sTmpResp;
   unsigned char* szTmpBuf = NULL;
   unsigned char* szTmpBuf64 = NULL;
-  unsigned char* szTmpChall = NULL;
-  int nSendBufferSize = 0;
-  int nReceiveBufferSize = 0;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating NTLM Authentication Attempt.", MODULE_NAME);
 
@@ -703,13 +696,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 
   /* Server should respond with an empty challenge, consisting simply of a "+" */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if ((strstr(bufReceive, "+OK") == NULL) && (strstr(bufReceive, "+ OK") == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ *OK.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] POP3 server did not respond with \"+ OK\" to AUTH NTLM request.", MODULE_NAME);
     return FAILURE;
@@ -717,21 +704,19 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   FREE(bufReceive);
 
   /* --- Send Base-64 encoded Type-1 message --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthRequest) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthRequest));
+  buildAuthRequest(&sTmpReq, 0, NULL, NULL);  
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthRequest) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthRequest) + 2);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpReq) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpReq) + 2);
 
-  buildAuthRequest((tSmbNtlmAuthRequest*)szTmpBuf, 0, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
   sprintf(bufSend, "%s\r\n", szTmpBuf64);
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
@@ -744,13 +729,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   /* Server should respond with a Base-64 encoded Type-2 challenge message. The challenge response format is 
      specified by RFC 1730 ("+", followed by a space, followed by the challenge message). */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strncmp(bufReceive, "+ ", 2) != 0)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Server did not send valid Type-2 challenge response.", MODULE_NAME);
     return FAILURE;
@@ -759,22 +738,19 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   szTmpBuf = ((char*)index(bufReceive, '\r'));
   szTmpBuf[0] = '\0';
 
-  szTmpChall = malloc( sizeof(tSmbNtlmAuthChallenge) );
-  memset(szTmpChall, 0, sizeof(tSmbNtlmAuthChallenge));
-
-  base64_decode(bufReceive + 2, szTmpChall);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, bufReceive + 2);
+  base64_decode(bufReceive + 2, (char *)&sTmpChall);
 
   FREE(bufReceive);
 
   /* --- Calculate and send Base-64 encoded Type 3 response --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthResponse) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthResponse));
+  buildAuthResponse(&sTmpChall, &sTmpResp, 0, szLogin, szPassword, _psSessionData->szDomain, NULL);
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthResponse) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthResponse) + 2);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpResp) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpResp) + 2);
 
-  buildAuthResponse((tSmbNtlmAuthChallenge*)szTmpChall, (tSmbNtlmAuthResponse*)szTmpBuf, 0, szLogin, szPassword, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
@@ -787,9 +763,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
-  FREE(szTmpChall);
   FREE(bufSend);
 
   /* Server should validate the response and indicate the result of authentication.
@@ -830,7 +804,6 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   {
     writeError(ERR_ERROR, "[%s] Failed during sending of authentication data.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     setPassResult(*psLogin, szPassword);
     return MSTATE_EXITING;
   }
@@ -838,10 +811,9 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
   else if (bufReceive[0] == '+')
@@ -850,6 +822,12 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
     nRet = MSTATE_EXITING;
   }
+  else if (strstr(bufReceive,"-ERR The specified authentication package is not supported.") != NULL) 
+  {
+    writeError(ERR_ERROR, "[%s] Server response: The specified authentication package is not supported.", MODULE_NAME);
+    (*psLogin)->iResult = LOGIN_RESULT_ERROR;
+    nRet = MSTATE_EXITING;
+  }
   else
   {
     if (_psSessionData->nMode == MODE_AS400)
