@@ -2,7 +2,7 @@
 **   SMB LAN Manager Password/HASH Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2008 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -94,8 +94,8 @@
 #define MODULE_NAME    "smbnt.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMB (LM/NTLM/LMv2/NTLMv2) sessions"
-#define MODULE_VERSION    "1.5"
-#define MODULE_VERSION_SVN "$Id: smbnt.c 1065 2009-02-06 23:05:03Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: smbnt.c 1228 2009-10-14 21:01:40Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -105,12 +105,6 @@
 #include <openssl/md4.h>
 #include <openssl/des.h>
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_SMB 139
 #define PORT_SMBNT 445
 #define WIN2000_NATIVEMODE 1
@@ -119,7 +113,7 @@
 #define HASH 4
 #define MACHINE 5
 #define LOCAL 6
-#define DOMAIN 7
+#define NTDOMAIN 7
 #define BOTH 8
 #define OTHER 9
 #define PLAINTEXT 10
@@ -167,12 +161,14 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
 int tryLogin(int hSocket, sLogin** login, _SMBNT_DATA* _psSessionData, char* szLogin, char* szPassword);
 int initModule(sLogin* login, _SMBNT_DATA *_psSessionData);
+char* parseFullyQualifiedUsername(_SMBNT_DATA *_psSessionData, char* szLogin);
 
 // Tell medusa how many parameters this module allows
 int getParamNumber()
@@ -265,7 +261,7 @@ int go(sLogin* logins, int argc, char *argv[])
   } 
   else 
   {
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
  
     psSessionData->authLevel = AUTH_NTLM;
     psSessionData->accntFlag = LOCAL;
@@ -288,7 +284,7 @@ int go(sLogin* logins, int argc, char *argv[])
         if (strcmp(pOpt, "LOCAL") == 0)
           psSessionData->accntFlag = LOCAL;
         else if (strcmp(pOpt, "DOMAIN") == 0)
-          psSessionData->accntFlag = DOMAIN;
+          psSessionData->accntFlag = NTDOMAIN;
         else if (strcmp(pOpt, "BOTH") == 0)
           psSessionData->accntFlag = BOTH;
         else
@@ -350,7 +346,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
@@ -358,13 +355,29 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
   char *bufReceive = NULL;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
   char *strtok_ptr = NULL, *pOpt = NULL, *pOptTmp = NULL;
-  char *pPass = NULL;
-  sUser *user = psLogin->psUser;
   char *szUser = NULL;
   sConnectParams params;
+  sCredentialSet *psCredSet = NULL;
+
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+ 
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {  
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+    szUser = parseFullyQualifiedUsername(_psSessionData, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
 
   memset(&params, 0, sizeof(sConnectParams));
   
@@ -375,59 +388,9 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
   
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  /* Split DOMAIN\USER style usernames */
-  if ( strstr(user->pUser, "\\") || strstr(user->pUser, "\\\\") )
-  {
-    if ((_psSessionData->accntFlag == DOMAIN) || (_psSessionData->accntFlag == OTHER))
-    {
-      writeError(ERR_NOTICE, "[%s] Using the DOMAIN\\USER format with the GROUP/GROUP_OTHER module options is redundant.", MODULE_NAME);
-    }
-
-    pOptTmp = malloc( strlen(user->pUser) + 1);
-    memset(pOptTmp, 0, strlen(user->pUser) + 1);
-    strncpy(pOptTmp, user->pUser, strlen(user->pUser));
-    writeError(ERR_DEBUG_MODULE, "Processing domain and username: %s", pOptTmp);
-
-    pOpt = strtok_r(pOptTmp, "\\", &strtok_ptr); 
-    strncpy((char *) _psSessionData->workgroup_other, pOpt, 16);
-    writeError(ERR_DEBUG_MODULE, "Processing domain: %s", _psSessionData->workgroup_other);
-    
-    pOpt = strtok_r(NULL, "\\", &strtok_ptr);
-    szUser = malloc(strlen(pOpt) + 1);
-    memset(szUser, 0, strlen(pOpt) + 1);
-    strncpy(szUser, pOpt, strlen(pOpt));
-    writeError(ERR_DEBUG_MODULE, "Processing username: %s", szUser);
-
-    FREE(pOptTmp);
-
-    _psSessionData->accntFlag = OTHER;
-  }
-  else
-  {
-    szUser = malloc(strlen(user->pUser) + 1); 
-    memset(szUser, 0, strlen(user->pUser) + 1);
-    strncpy(szUser, user->pUser, strlen(user->pUser));
-    writeError(ERR_DEBUG_MODULE, "Processing username: %s", szUser);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         // Already have an open socket - close it
@@ -451,8 +414,6 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         {
           writeError(ERR_ERROR, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          free(_psSessionData);
           return FAILURE;
         }
 
@@ -461,8 +422,6 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         if (NBSSessionRequest(hSocket, _psSessionData) < 0) {
           writeError(ERR_ERROR, "Session Setup Failed with host: %s. Is the server service running?", psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          free(_psSessionData);
           return FAILURE;
         }
         
@@ -470,8 +429,6 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         {
           writeError(ERR_ERROR, "SMB Protocol Negotiation Failed with host: %s", psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          free(_psSessionData);
           return FAILURE;
         }
         else {
@@ -480,15 +437,39 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, szUser, pPass);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, szUser, psCredSet->pPass);
+        
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              FREE(szUser);
+              szUser = parseFullyQualifiedUsername(_psSessionData, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module (%d) state %d host: %s", MODULE_NAME, psLogin->iId, nState, psLogin->psServer->pHostIP);
@@ -496,19 +477,61 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
-        free(_psSessionData);
         return FAILURE;
     }  
   }
+
+  writeError(ERR_DEBUG_MODULE, "[%s] Exiting module...", MODULE_NAME);
  
-  psLogin->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   FREE(szUser);
-  free(_psSessionData);
   return SUCCESS;
 }
 
 /* SMBNT Specific Functions */
+ 
+/* Split DOMAIN\USER style usernames */
+char* parseFullyQualifiedUsername(_SMBNT_DATA *_psSessionData, char* szLogin)
+{
+  char *strtok_ptr = NULL, *pOpt = NULL, *pOptTmp = NULL;
+  char *szUser = NULL;
+ 
+  if ( strstr(szLogin, "\\") || strstr(szLogin, "\\\\") )
+  {
+    if ((_psSessionData->accntFlag == NTDOMAIN) || (_psSessionData->accntFlag == OTHER))
+    {
+      writeError(ERR_NOTICE, "[%s] Using the DOMAIN\\USER format with the GROUP/GROUP_OTHER module options is redundant.", MODULE_NAME);
+    }
+
+    pOptTmp = malloc( strlen(szLogin) + 1);
+    memset(pOptTmp, 0, strlen(szLogin) + 1);
+    strncpy(pOptTmp, szLogin, strlen(szLogin));
+    writeError(ERR_DEBUG_MODULE, "Processing domain and username: %s", pOptTmp);
+
+    pOpt = strtok_r(pOptTmp, "\\", &strtok_ptr); 
+    strncpy((char *) _psSessionData->workgroup_other, pOpt, 16);
+    writeError(ERR_DEBUG_MODULE, "Processing domain: %s", _psSessionData->workgroup_other);
+    
+    pOpt = strtok_r(NULL, "\\", &strtok_ptr);
+    szUser = malloc(strlen(pOpt) + 1);
+    memset(szUser, 0, strlen(pOpt) + 1);
+    strncpy(szUser, pOpt, strlen(pOpt));
+    writeError(ERR_DEBUG_MODULE, "Processing username: %s", szUser);
+
+    FREE(pOptTmp);
+
+    _psSessionData->accntFlag = OTHER;
+  }
+  else
+  {
+    szUser = malloc(strlen(szLogin) + 1); 
+    memset(szUser, 0, strlen(szLogin) + 1);
+    strncpy(szUser, szLogin, strlen(szLogin));
+    writeError(ERR_DEBUG_MODULE, "Processing username: %s", szUser);
+  }
+
+  return szUser;
+}
 
 static unsigned char Get7Bits(unsigned char *input, int startBit)
 {
@@ -1697,7 +1720,6 @@ void showUsage()
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
   writeVerbose(VB_NONE, "** Module was not properly built. Is OPENSSL installed correctly? **");
   writeVerbose(VB_NONE, "");
-  return FAILURE;
 }
 
 int go(sLogin* logins, int argc, char *argv[])
