@@ -2,7 +2,7 @@
 **   SMTP VRFY Account Enumeration Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -34,17 +34,11 @@
 #define MODULE_NAME    "smtp-vrfy.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for enumerating accounts via SMTP VRFY"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: smtp-vrfy.c 976 2008-09-26 20:15:55Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: smtp-vrfy.c 1290 2010-01-08 17:02:07Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_SMTP 25
 #define BUF_SIZE 300
 #define RECEIVE_DELAY_1 20 * 1000000
@@ -60,7 +54,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -92,7 +87,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -157,7 +151,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
@@ -168,29 +163,31 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
   int nReceiveBufferSize = 0;
   int nStatus = FAILURE;
   int nBannerStatus = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   params.nPort = PORT_SMTP;
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
   if (_psSessionData->szEHLO == NULL)
   {
     _psSessionData->szEHLO = malloc(7);
@@ -198,9 +195,9 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
     sprintf(_psSessionData->szEHLO, "MEDUSA");
   }
 
-  writeError(ERR_DEBUG, "[%s] Set EHLO value: %s", MODULE_NAME, _psSessionData->szEHLO);
+  writeError(ERR_DEBUG_MODULE, "[%s] Set EHLO value: %s", MODULE_NAME, _psSessionData->szEHLO);
 
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
     switch(nState)
     {
@@ -217,7 +214,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
@@ -249,7 +245,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         {
           writeError(ERR_DEBUG_MODULE, "[%s] Server did not respond with '220' code. Exiting...", MODULE_NAME);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           nState = MSTATE_EXITING;
           return FAILURE;
           break;
@@ -269,16 +264,38 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, pPass, user->pUser);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         sayQUIT(hSocket);
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -286,12 +303,11 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  psLogin->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -388,7 +404,7 @@ int sayQUIT(int hSocket)
   
   421 Error: too many errors
 */
-int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char* szDomain, char* szAccount)
+int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char* szAccount, char* szDomain)
 {
   int iRet;
   unsigned char bufSend[BUF_SIZE];
@@ -396,9 +412,18 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   int nReceiveBufferSize = 0;
 
   /* send helo string */
-  writeError(ERR_DEBUG_MODULE, "[%s] Sending: VRFY %.250s@%.250s\r\n", MODULE_NAME, szAccount, szDomain);
   memset(bufSend, 0, sizeof(bufSend));
-  sprintf(bufSend, "VRFY %.250s@%.250s\r\n", szAccount, szDomain);
+  
+  if (strlen(szDomain) > 0)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] Sending: VRFY %.250s@%.250s\r\n", MODULE_NAME, szAccount, szDomain);
+    sprintf(bufSend, "VRFY %.250s@%.250s\r\n", szAccount, szDomain);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] Sending: VRFY %.250s\r\n", MODULE_NAME, szAccount);
+    sprintf(bufSend, "VRFY %.250s\r\n", szAccount);
+  }
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
