@@ -2,7 +2,7 @@
 **   SMTP Authentication Daemon Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 pMonkey
+**    Copyright (C) 2009 pMonkey
 **    pMonkey <pmonkey@foofus.net>
 **
 **    Copyright (C) 2008 JoMo-Kun
@@ -32,25 +32,19 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include "module.h"
 #include "ntlm.h"
 
 #define MODULE_NAME    "smtp.mod"
 #define MODULE_AUTHOR  "pmonkey@foofus.net"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMTP Authentication with TLS"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: smtp.c 987 2008-10-10 17:43:41Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: smtp.c 1259 2009-11-16 04:19:46Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
-#define PORT_SMTPAUTH 25
+#define PORT_SMTP 25
+#define PORT_SMTPS 465
 
 #define RECEIVE_DELAY_1 20 * 1000000
 #define RECEIVE_DELAY_2 0.5 * 1000000
@@ -63,6 +57,7 @@
 typedef struct __MODULE_DATA {
   char *szEHLO;
   int nAuthType;
+  char* szDomain;
 } _MODULE_DATA;
 
 
@@ -71,7 +66,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -114,6 +110,11 @@ void showUsage()
   writeVerbose(VB_NONE, " AUTH:? (Authentication Type (PLAIN/LOGIN/NTLM). Default: automatic)");
   writeVerbose(VB_NONE, "    Module will query service for accepted methods via an \"AUTH\" request.");
   writeVerbose(VB_NONE, "    PLAIN, LOGIN, and NTLM authentication methods are supported.");
+  writeVerbose(VB_NONE, "  DOMAIN:? [optional]");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "The DOMAIN option should supply the specified domain appropriately,");
+  writeVerbose(VB_NONE, "regardless of authentication type. The domain can also be supplied ");
+  writeVerbose(VB_NONE, "via the username field, but the format appears to differ by auth type.");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "  Usage example: ");
   writeVerbose(VB_NONE, "    \"medusa -M pop3 -m AUTH:NTLM -U accounts.txt -p password\"");
@@ -162,6 +163,20 @@ int go(sLogin* logins, int argc, char *argv[])
         else
           writeError(ERR_WARNING, "Invalid value for method AUTH.");
       }
+      else if (strcmp(pOpt, "DOMAIN") == 0)
+      {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if ( pOpt )
+        {
+          psSessionData->szDomain = malloc(strlen(pOpt) + 1);
+          memset(psSessionData->szDomain, 0, strlen(pOpt) + 1);
+          strncpy((char *) psSessionData->szDomain, pOpt, strlen(pOpt));
+        }
+        else
+          writeError(ERR_WARNING, "Method DOMAIN requires value to be set.");
+      }
       else if (strcmp(pOpt, "EHLO") == 0)
       {
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
@@ -185,28 +200,42 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive = NULL;
-  int nReceiveBufferSize = 0;
-  int i = 0;
-  char *pPass = NULL;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
-  memset(&params, 0, sizeof(sConnectParams));
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
 
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
+  memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
+  else if (psLogin->psServer->psHost->iUseSSL > 0)
+    params.nPort = PORT_SMTPS;
   else
-    params.nPort = PORT_SMTPAUTH;
-
-  params.nSSLVersion = 9; /* Force use of TLS */
+    params.nPort = PORT_SMTP;
   initConnectionParams(psLogin, &params);
 
   /* set EHLO, if not specified by user */
@@ -217,22 +246,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
     sprintf(_psSessionData->szEHLO, "gerg");
   }
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
     switch(nState)
     {
@@ -240,21 +254,21 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         if (hSocket > 0)
           medusaDisconnect(hSocket);
   
-        hSocket = medusaConnect(&params);
+        if (psLogin->psServer->psHost->iUseSSL > 0)
+          hSocket = medusaConnectSSL(&params);
+        else
+          hSocket = medusaConnect(&params);
+
         if (hSocket < 0) 
         {
           writeError(ERR_NOTICE, "[%s] failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          FREE(_psSessionData);
           return FAILURE;
         }
 
         if (initConnection(_psSessionData, hSocket, &params) == FAILURE)
         {
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
-          FREE(_psSessionData);
           return FAILURE;
         }
 
@@ -263,15 +277,49 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
-        if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        if ( medusaCheckSocket(hSocket) )
+        {
+          nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
+          if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
+          {
+            if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+            {
+              writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else
+            {
+              if (psCredSet->iStatus == CREDENTIAL_DONE)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+                nState = MSTATE_EXITING;
+              }
+              else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+              {
+                writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+                nState = MSTATE_NEW;
+              }
+              else
+                writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+            }
+          }
+        }
+        else
+        {
+          writeError(ERR_NOTICE, "[%s] Socket is no longer valid. Server likely dropped connection. Establishing new session.", MODULE_NAME);
+          nState = MSTATE_NEW;
+
+          if (hSocket > 0)
+            medusaDisconnect(hSocket);
+          hSocket = -1;
+        }
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -279,14 +327,11 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
-        FREE(_psSessionData);
         return FAILURE;
     }  
   }
 
-  psLogin->iStatus = LOGIN_DONE;
-  FREE(_psSessionData);
+  FREE(psCredSet);
   return SUCCESS;
 }
        
@@ -303,30 +348,12 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   /* Retrieve SMTP banner */
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving SMTP banner.", MODULE_NAME);  
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveLine(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strncmp(bufReceive, "220", 3) != 0)
-  {
-    writeError(ERR_ERROR, "[%s] Is this an SMTP server?: %s.", MODULE_NAME, bufReceive);
+    writeError(ERR_DEBUG_MODULE, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
     FREE(bufReceive);  
     return FAILURE;
   }
-  FREE(bufReceive);
- 
-  /* Test if more data is waiting */ 
-  if (medusaDataReadyTimed(hSocket, 0, 20000) > 0)
-  {
-    bufReceive = medusaReceiveLineDelay(hSocket, &nReceiveBufferSize, RECEIVE_DELAY_1, RECEIVE_DELAY_2);
-    if (bufReceive != NULL)
-    {
-      bufReceive[nReceiveBufferSize] = 0;  // Make certain buffer is null-terminated
-      FREE(bufReceive);
-    }
-  }
  
   /* Send greeting to SMTP server */
   writeError(ERR_DEBUG_MODULE, "[%s] Sending SMTP EHLO greeting.", MODULE_NAME);  
@@ -344,15 +371,15 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   FREE(bufSend); 
  
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "250-.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] failed: Server did not respond with '250'. Exiting...", MODULE_NAME);
+    FREE(bufReceive);
     return FAILURE;
   }
 
-  /* If server supports STARTTLS, we use it */
-  if (strstr(bufReceive, "250-STARTTLS") != NULL)
+  /* If server supports STARTTLS and we are not already within a SSL connection, let's use it. */
+  if ((params->nUseSSL == 0) && (strstr(bufReceive, "STARTTLS") != NULL))
   {
     FREE(bufReceive);
   
@@ -370,16 +397,17 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
     FREE(bufSend);
   
     nReceiveBufferSize = 0;
-    bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-    if (bufReceive == NULL)
+    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
     {
-      writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
+      writeError(ERR_ERROR, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
+      FREE(bufReceive);
       return FAILURE;
     }
-    else if (strstr(bufReceive, "220") != NULL)
+    else
     {
       FREE(bufReceive);
-      
+     
+      params->nSSLVersion = 3.1; /* Force the use of TLSv1 */ 
       if (medusaConnectSocketSSL(params, hSocket) < 0)
       {
         writeError(ERR_ERROR, "[%s] Failed to establish SSLv3 connection.", MODULE_NAME);
@@ -402,10 +430,10 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       FREE(bufSend); 
  
       nReceiveBufferSize = 0;
-      bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-      if (bufReceive == NULL)
+      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
       {
-        writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
+        writeError(ERR_ERROR, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
+        FREE(bufReceive);
         return FAILURE;
       }
     }
@@ -487,15 +515,10 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a 334 response code */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  else if (strstr(bufReceive, "334 ") == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH PLAIN request.", MODULE_NAME);
+    FREE(bufReceive);
     return FAILURE;
   }
 
@@ -544,6 +567,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   unsigned char* szPrompt = NULL;
   unsigned char* szTmpBuf = NULL;
   unsigned char* szTmpBuf2 = NULL;
+  unsigned char* szLoginDomain = NULL;
   int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
   int nRet = SUCCESS;
@@ -563,20 +587,14 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded username prompt */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-
-  if ((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH LOGIN request.", MODULE_NAME);
+    FREE(bufReceive);
     return FAILURE;
   }
 
-  if ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL)
+  if (((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server sent unexpected response to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
@@ -597,10 +615,25 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   /* --- Send username --- */
 
   /* Base64 encoded value can be up to 2x+2 original text. Leave additional space for "\r\n" and NULL */
-  bufSend = malloc((2 * strlen(szLogin) + 2) + 2 + 1);
-  memset(bufSend, 0, (2 * strlen(szLogin) + 2) + 2 + 1);
-  base64_encode(szLogin, strlen(szLogin), bufSend);
+  if (_psSessionData->szDomain)
+  {
+    /* DOMAIN\USERNAME */
+    szLoginDomain = malloc(strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
+    memset(szLoginDomain, 0, strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
+    sprintf(szLoginDomain, "%s\\%s", _psSessionData->szDomain, szLogin); 
+  }
+  else
+    szLoginDomain = szLogin;
+
+  writeError(ERR_DEBUG_MODULE, "[%s] Sending authenticate login value: %s %s", MODULE_NAME, szLoginDomain, szPassword);
+  
+  bufSend = malloc((2 * strlen(szLoginDomain) + 2) + 2 + 1);
+  memset(bufSend, 0, (2 * strlen(szLoginDomain) + 2) + 2 + 1);
+  base64_encode(szLoginDomain, strlen(szLoginDomain), bufSend);
   strncat(bufSend, "\r\n", 2);
+  
+  if (_psSessionData->szDomain)
+    FREE(szLoginDomain);
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
@@ -609,20 +642,14 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded password prompt */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH LOGIN request.", MODULE_NAME);
+    FREE(bufReceive);  
     return FAILURE;
   }
 
-  if ((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH LOGIN request.", MODULE_NAME);
-    return FAILURE;
-  }
-    
-  if ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL)
+  if (((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server sent unexpected response to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
@@ -663,30 +690,31 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 {
   unsigned char* bufSend = NULL;
   unsigned char* bufReceive = NULL;
+  int nReceiveBufferSize = 0;
+  int nSendBufferSize = 0;
+  tSmbNtlmAuthRequest   sTmpReq;
+  tSmbNtlmAuthChallenge sTmpChall;
+  tSmbNtlmAuthResponse  sTmpResp;
   unsigned char* szTmpBuf = NULL;
   unsigned char* szTmpBuf64 = NULL;
-  unsigned char* szTmpChall = NULL;
-  int nSendBufferSize = 0;
-  int nReceiveBufferSize = 0;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating NTLM Authentication Attempt.", MODULE_NAME);
 
   /* --- Send Base-64 encoded Type-1 message --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthRequest) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthRequest));
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthRequest) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthRequest) + 2);
+  buildAuthRequest(&sTmpReq, 0, NULL, NULL);
 
-  buildAuthRequest((tSmbNtlmAuthRequest*)szTmpBuf, 0, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpReq) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpReq) + 2);
+
+  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(10 + nSendBufferSize + 1);
   memset(bufSend, 0, 10 + nSendBufferSize + 1);
   sprintf(bufSend, "AUTH NTLM %s\r\n", szTmpBuf64);
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
 
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
@@ -699,37 +727,29 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   /* Server should respond with a Base-64 encoded Type-2 challenge message. The challenge response format is 
      "334", followed by a space, followed by the challenge message. */
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
-  
-  if ((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Server did not send valid Type-2 challenge response.", MODULE_NAME);
+    FREE(bufReceive);
     return FAILURE;
   }
-  szTmpBuf += 4;
 
-  szTmpChall = malloc( sizeof(tSmbNtlmAuthChallenge) );
-  memset(szTmpChall, 0, sizeof(tSmbNtlmAuthChallenge));
+  szTmpBuf = ((char*)index(bufReceive, '\r'));
+  szTmpBuf[0] = '\0';
 
-  writeError(ERR_DEBUG_MODULE, "[%s] Server Type-2 NTLM challenge: %s", MODULE_NAME, szTmpBuf);
-  base64_decode(szTmpBuf, szTmpChall);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, bufReceive + 4);
+  base64_decode(bufReceive + 4, (char *)&sTmpChall);
 
   FREE(bufReceive);
 
   /* --- Calculate and send Base-64 encoded Type 3 response --- */
-  szTmpBuf = malloc( sizeof(tSmbNtlmAuthResponse) );
-  memset(szTmpBuf, 0, sizeof(tSmbNtlmAuthResponse));
+  buildAuthResponse(&sTmpChall, &sTmpResp, 0, szLogin, szPassword, _psSessionData->szDomain, NULL);
 
-  szTmpBuf64 = malloc( 2 * sizeof(tSmbNtlmAuthResponse) + 2);
-  memset(szTmpBuf64, 0, 2 * sizeof(tSmbNtlmAuthResponse) + 2);
+  szTmpBuf64 = malloc(2 * SmbLength(&sTmpResp) + 2);
+  memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpResp) + 2);
 
-  buildAuthResponse((tSmbNtlmAuthChallenge*)szTmpChall, (tSmbNtlmAuthResponse*)szTmpBuf, 0, szLogin, szPassword, NULL, NULL);
-  base64_encode(szTmpBuf, sizeof(tSmbNtlmAuthResponse), szTmpBuf64);
+  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
+  writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
@@ -742,9 +762,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  FREE(szTmpBuf);
   FREE(szTmpBuf64);
-  FREE(szTmpChall);
   FREE(bufSend);
 
   return SUCCESS;
@@ -785,11 +803,11 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^\\d+ .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
+    writeError(ERR_ERROR, "[%s] Unknown SMTP server response: %s", MODULE_NAME, bufReceive);
+    (*psLogin)->iResult = LOGIN_RESULT_ERROR;
+    nRet = MSTATE_EXITING;
   }
 
   if (strstr(bufReceive,"235 ") != NULL)
@@ -801,13 +819,20 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   /* 435 Unable to authenticate at present: Authentication Failure */
   else if (strstr(bufReceive,"435 ") != NULL)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed.", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (435).", MODULE_NAME);
+    (*psLogin)->iResult = LOGIN_RESULT_FAIL;
+    nRet = MSTATE_RUNNING;
+  }
+  /* GroupWise - 501 Authentication failed! */
+  else if (strstr(bufReceive,"501 ") != NULL)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (501).", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     nRet = MSTATE_RUNNING;
   }
   else if (strstr(bufReceive,"535 ") != NULL)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed.", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (535).", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     nRet = MSTATE_RUNNING;
   }
