@@ -2,7 +2,7 @@
 **   SNMPv1/2C Community String Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -32,23 +32,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include "module.h"
 
 #define MODULE_NAME    "snmp.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SNMP Community Strings"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: snmp.c 569 2006-07-31 21:01:06Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: snmp.c 1225 2009-10-13 18:55:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_SNMP 161
 
 #define SNMP_VER_V1 1
@@ -59,7 +52,7 @@
 #define SNMP_SUCCESS_WRITE 2
 
 #define SEND_DELAY 200 /* Delay between sending SNMP requests (usec) */
-#define RECEIVE_DELAY 5*1000000 /* Rresponse wait time (usec) */
+#define RECEIVE_DELAY 5*1000000 /* Response wait time (usec) */
 
 typedef struct __SNMP_DATA {
   int nVersion;
@@ -73,7 +66,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -105,7 +99,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -228,7 +221,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
@@ -236,28 +230,40 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
   int i = 0, nPassCount, nPassCountWrite;
-  char *pPass;
   char **arrszPassList = NULL;
   char **arrszPassListWrite = NULL;
   char *szLocation = NULL;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
+  sUser *psUser = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   params.nPort = PORT_SNMP;
   initConnectionParams(psLogin, &params);
 
   writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
 
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         if (hSocket > 0)
@@ -268,7 +274,6 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
@@ -276,11 +281,10 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = sendRead(hSocket, _psSessionData, pPass);
+        nState = sendRead(hSocket, _psSessionData, psCredSet->pPass);
         if (nState == FAILURE)
         {
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
@@ -290,16 +294,43 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
 
         /* initially set all passwords as invalid -- we don't know their validity yet */
         psLogin->iResult = LOGIN_RESULT_FAIL;
-        setPassResult(psLogin, pPass);
+        setPassResult(psLogin, psCredSet->pPass);
         
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+
+              /* Medusa has exhausted all credential sets and reset psLogin->psUser to NULL. This 
+                 creates issues as we haven't actually received the responses yet from our SNMP
+                 queries. Our solution is to create a temporary sUser structure, which allows us
+                 to report on successful community strings via normal methods. */ 
+              psUser = malloc(sizeof(sUser));
+              memset(psUser, 0, sizeof(sUser));
+              psLogin->psUser = psUser; 
+
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_RUNNING;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
-        if (hSocket > 0)
-          medusaDisconnect(hSocket);
-        hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -307,7 +338,6 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
@@ -316,7 +346,6 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
   if (receiveRequest(hSocket, _psSessionData, &nPassCount, &arrszPassList, &szLocation) == FAILURE)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Failed to find valid READ community string.", MODULE_NAME);
-    psLogin->iStatus = LOGIN_DONE;
     return SUCCESS;
   }
 
@@ -361,10 +390,14 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
     FREE(arrszPassList[i]);
     FREE(szLocation);
   }
+        
+  if (hSocket > 0)
+    medusaDisconnect(hSocket);
 
+  FREE(psUser);
+  FREE(psCredSet);
   FREE(arrszPassList);
   FREE(arrszPassListWrite);
-  psLogin->iStatus = LOGIN_DONE;
   return SUCCESS;
 }
 
