@@ -2,7 +2,7 @@
 **   SSH v2 Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -35,8 +35,8 @@
 #define MODULE_NAME    "ssh.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SSH v2 sessions"
-#define MODULE_VERSION    "1.0.2"
-#define MODULE_VERSION_SVN "$Id: ssh.c 955 2008-08-20 16:04:44Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: ssh.c 1295 2010-02-04 19:15:40Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBSSH2_WARNING "No usable LIBSSH2. Module disabled."
@@ -68,7 +68,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -119,6 +120,17 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_SSH2_DATA));
   memset(psSessionData, 0, sizeof(_SSH2_DATA));
 
+#ifdef LIBSSH2_GCRYPT
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "The default build of Libssh2 is to use OpenSSL for crypto. Several Linux");
+  writeVerbose(VB_NONE, "distributions (e.g. Debian, Ubuntu) build it to use Libgcrypt. Unfortunately,");
+  writeVerbose(VB_NONE, "the implementation within Libssh2 of libgcrypt appears to be broken and is");
+  writeVerbose(VB_NONE, "not thread safe. If you run multiple concurrent Medusa SSH connections, you");
+  writeVerbose(VB_NONE, "are likely to experience segmentation faults. Please help Libssh2 fix this");
+  writeVerbose(VB_NONE, "issue or encourage your distro to use the default Libssh2 build options.");
+  writeVerbose(VB_NONE, "");
+#endif
+
   if ( !(0 <= argc <= 1) )
   {
     // Show usage information
@@ -160,7 +172,8 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 // void *name(size_t count, void **abstract)
@@ -186,11 +199,8 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
   int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
   LIBSSH2_SESSION *session = NULL;
   char *pErrorMsg;
@@ -211,24 +221,28 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
  
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
+  /* Retrieve next available credential set to test */
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
   }
-  else 
+  else if (psCredSet->psUser)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME, psLogin->psServer->pHostIP);
+    nState = MSTATE_COMPLETE;
   }
 
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         /*
@@ -237,21 +251,19 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         */
         if (session)
         {
-          writeError(ERR_DEBUG_MODULE, "%s: Destroying previous SSH session structure: Host: %s User: %s", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
+          writeError(ERR_DEBUG_MODULE, "%s: Destroying previous SSH session structure: Host: %s User: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
           libssh2_session_free(session);
         }
         
         session = libssh2_session_init_ex(ssh2_init_alloc_cb, ssh2_init_free_cb, ssh2_init_realloc_cb, &ssh2_session_data);
         if (!session)
         {
-          writeError(ERR_ERROR, "%s: Failed initiating SSH session: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser, pPass);
+          writeError(ERR_ERROR, "%s: Failed initiating SSH session: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }        
 
         /* Set client SSH banner */
-           
         if (_psSessionData->szBannerMsg) {
           if ((strncmp(_psSessionData->szBannerMsg, "SSH-2.0-", 8) != 0) || (strlen(_psSessionData->szBannerMsg) > 253)) {
             writeError(ERR_ERROR, "[%s] The selected banner could be rejected be some SSH servers. The banner should begin with \"SSH-2.0-\" and be shorter than 253 characters.", MODULE_NAME);
@@ -267,7 +279,8 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         if ( libssh2_banner_set(session, _psSessionData->szBannerMsg) ) {
            writeError(ERR_DEBUG_MODULE, "Failed to set libssh banner.");
         }
-        
+       
+        /* Initiate SSH session connection - retry if necessary */ 
         writeError(ERR_DEBUG_MODULE, "Attempting to initiate SSH session.");
         for (i = 1; i <= psLogin->psServer->psHost->iRetries + 1; i++) {
           if (hSocket > 0) {
@@ -280,12 +293,11 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
             if (session)
               libssh2_session_free(session);
             psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-            psLogin->iStatus = LOGIN_FAILED;
             return FAILURE;
           }
           
           if (libssh2_session_startup(session, hSocket)) {
-            writeError(ERR_ERROR, "%s: Failed establishing SSH session (%d/%d): Host: %s User: %s Pass: %s", MODULE_NAME, i, psLogin->psServer->psHost->iRetries + 1, psLogin->psServer->pHostIP, user->pUser, pPass);
+            writeError(ERR_ERROR, "%s: Failed establishing SSH session (%d/%d): Host: %s User: %s Pass: %s", MODULE_NAME, i, psLogin->psServer->psHost->iRetries + 1, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
           
             libssh2_session_last_error(session, &pErrorMsg, &iErrorMsg, 1);
             if ( (pErrorMsg) && (strstr(pErrorMsg, "Unable to exchange encryption keys")) ) {
@@ -297,7 +309,12 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
             }
 
             if (i == psLogin->psServer->psHost->iRetries + 1) {
-              writeError(ERR_ERROR, "%s: Failed establishing SSH session. The following credentials were NOT tested: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser, pPass);
+              
+              if (addMissedCredSet(psLogin, psCredSet) == SUCCESS)
+                writeError(ERR_ERROR, "%s: Failed establishing SSH session. The following credentials have been added to the missed queue for later testing: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
+              else
+                writeError(ERR_ERROR, "%s: Failed establishing SSH session. The following credentials were NOT tested: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
+
               libssh2_session_disconnect(session, "");
               if (session)
                 libssh2_session_free(session);
@@ -305,7 +322,6 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
                 medusaDisconnect(hSocket);
               hSocket = -1;
               psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-              psLogin->iStatus = LOGIN_FAILED;
               return FAILURE;
             }
         
@@ -318,44 +334,70 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        ssh2_session_data.pPass = pPass;
+        ssh2_session_data.pPass = psCredSet->pPass;
         ssh2_session_data.iAnswerCount = 0;
-        nState = tryLogin(_psSessionData, hSocket, session, &psLogin, user->pUser, pPass);
+        nState = tryLogin(_psSessionData, hSocket, session, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
+
         break;
       case MSTATE_EXITING:
-        pPass = NULL;
+        if (session)
+        {
+          libssh2_session_disconnect(session, "");
+          libssh2_session_free(session);
+        }
+
+        if (hSocket > 0)
+        {
+          medusaDisconnect(hSocket);
+          hSocket = -1;
+        }
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
-        libssh2_session_disconnect(session, "");
+
         if (session)
+        {
+          libssh2_session_disconnect(session, "");
           libssh2_session_free(session);
+        }
+
         if (hSocket > 0)
+        {
           medusaDisconnect(hSocket);
-        hSocket = -1;
+          hSocket = -1;
+        }
+
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
-  
-  if (session)
-  {
-    libssh2_session_disconnect(session, "");
-    libssh2_session_free(session);
-  }
-  
-  if (hSocket > 0)
-  {
-    medusaDisconnect(hSocket);
-    hSocket = -1;
-  }
- 
-  if (psLogin->iStatus != LOGIN_FAILED)
-    psLogin->iStatus = LOGIN_DONE;
 
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -442,7 +484,6 @@ int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session,
   else {
     writeError(ERR_ERROR, "Failed to retrieve supported authentication modes. Aborting...");
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-    (*psLogin)->iStatus = LOGIN_FAILED;
     iRet = MSTATE_EXITING;
   }
  
@@ -477,11 +518,9 @@ int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session,
         iRet = MSTATE_EXITING;
       }
       break;
-
     default:
       writeError(ERR_ERROR, "No supported authentication methods located.");
       (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
-      (*psLogin)->iStatus = LOGIN_FAILED;
       iRet = MSTATE_EXITING;
       break;
   }
