@@ -1,6 +1,6 @@
 /***************************************************************************
  *   telnet.c                                                              *
- *   Copyright (C) 2006 by fizzgig                                         *
+ *   Copyright (C) 2009 by fizzgig                                         *
  *   fizzgig@foofus.net                                                    *
  *                                                                         *
  *   Implementation of a telnet brute force module for                     *
@@ -44,19 +44,13 @@
 #include <arpa/telnet.h>
 #include "module.h"
 
-#define  MODULE_NAME    "telnet.mod"
+#define MODULE_NAME    "telnet.mod"
 #define MODULE_AUTHOR  "fizzgig <fizzgig@foofus.net>"
-#define  MODULE_SUMMARY_USAGE  "Brute force module for telnet sessions"
-#define MODULE_VERSION    "1.2.2"
-#define MODULE_VERSION_SVN "$Id: telnet.c 1040 2009-01-19 16:43:59Z jmk $"
+#define MODULE_SUMMARY_USAGE  "Brute force module for telnet sessions"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: telnet.c 1271 2009-11-25 00:24:58Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PORT_TELNET 23
 #define PORT_TELNETS 992
 
@@ -80,8 +74,6 @@ const char* KNOWN_PROMPTS = ">#$%/?";  // Each character represents a known teln
 const int KNOWN_PWD_SIZE = 4;  // Make sure to keep this in sync with the size of the array below!!
 const char* KNOWN_PWD_PROMPTS[] = { "assword", "asscode", "ennwort", "ASSWORD" };  // Complete/partial lines that indicate a password request
 
-//const int KNOWN_LOGIN_SIZE = 4;  // Make sure to keep this in sync with the size of the array below!!
-//const char* KNOWN_LOGIN_PROMPTS[] = { "login:", "last login", "sername:", "User" }; // Complete/partial lines that request a user name
 const int KNOWN_LOGIN_SIZE = 3;  // Make sure to keep this in sync with the size of the array below!!
 const char* KNOWN_LOGIN_PROMPTS[] = { "login:", "sername:", "User" }; // Complete/partial lines that request a user name
 
@@ -90,7 +82,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -141,7 +134,6 @@ void showUsage()
 int go(sLogin* logins, int argc, char *argv[])
 {
   int i;
-
   char *strtok_ptr, *pOpt, *pOptTmp;
   _MODULE_DATA *psSessionData;
 
@@ -185,65 +177,66 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }
 
+  FREE(psSessionData);
   return SUCCESS;
 }
 
-int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
+int initModule(sLogin* _psLogin, _MODULE_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
   char* bufReceive;
   int nReceiveBufferSize = 0, nFoundPrompt = PROMPT_UNKNOWN;
   int i = 0;
-  char *pPass;
-  int nPort, nOptions = 0;
-  sUser* user = login->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, _psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
-  if (login->psServer->psAudit->iPortOverride > 0)
-    params.nPort = login->psServer->psAudit->iPortOverride;
-  else if (login->psServer->psHost->iUseSSL > 0)
+  if (_psLogin->psServer->psAudit->iPortOverride > 0)
+    params.nPort = _psLogin->psServer->psAudit->iPortOverride;
+  else if (_psLogin->psServer->psHost->iUseSSL > 0)
     params.nPort = PORT_TELNETS;
   else
     params.nPort = PORT_TELNET;
-  initConnectionParams(login, &params);
+  initConnectionParams(_psLogin, &params);
 
-  if (user != NULL)
+  while (nState != MSTATE_COMPLETE)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s:%d, user: '%s'", MODULE_NAME, login->psServer->pHostIP, params.nPort, user->pUser);
-  }
-  else
-  {
-    writeError(ERR_DEBUG_MODULE, "null user");
-  }
-
-  pPass = getNextPass(login->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, login->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
-  {
-    switch(nState)
+    switch (nState)
     {
     case MSTATE_NEW:
       // Already have an open socket - close it
       if (hSocket > 0)
         medusaDisconnect(hSocket);
 
-      if (login->psServer->psHost->iUseSSL > 0)
+      if (_psLogin->psServer->psHost->iUseSSL > 0)
         hSocket = medusaConnectSSL(&params);
       else 
         hSocket = medusaConnect(&params);
       
       if (hSocket <= 0)
       {
-        writeError(ERR_ERROR, "[%s] Failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, login->psServer->pHostIP);
-        login->iResult = LOGIN_RESULT_UNKNOWN;
-        login->iStatus = LOGIN_FAILED;
-        setPassResult(login, pPass);
+        writeError(ERR_ERROR, "[%s] Failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, _psLogin->psServer->pHostIP);
+        _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+        setPassResult(_psLogin, psCredSet->pPass);
         return FAILURE;
       }
 
@@ -260,9 +253,8 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
       if (bufReceive == NULL)
       {
         writeError(ERR_ERROR, "[%s] null response was unexpected from a telnet server (is one running?)", MODULE_NAME);
-        login->iResult = LOGIN_RESULT_UNKNOWN;
-        login->iStatus = LOGIN_FAILED;
-        setPassResult(login, pPass);
+        _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+        setPassResult(_psLogin, psCredSet->pPass);
         return FAILURE;
       }
 
@@ -281,7 +273,7 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
 
           if (_psSessionData->nMode == MODE_AS400)
           {
-            if (strcasestr(bufReceive, "Sign On") != NULL)
+            if (strcasestr(bufReceive, (char *)"Sign On") != '\0')
             {
               writeError(ERR_INFO, "[%s] Detected AS/400 Sign On Screen.", MODULE_NAME);
               nFoundPrompt = PROMPT_LOGIN_PASSWORD;
@@ -310,7 +302,7 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
             // Look for known login prompts
             for (i = 0; i < KNOWN_LOGIN_SIZE; i++)
             {
-              if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != NULL)
+              if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != '\0')
               {
                 // Do we have a prompt?
                 writeError(ERR_DEBUG_MODULE, "Found login prompt...");
@@ -322,14 +314,14 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
             /* Some systems do not provide a login prompt and go right to password */
             for (i = 0; i < KNOWN_PWD_SIZE; i++)
             {
-              if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != NULL)
+              if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != '\0')
               {
                 // Do we have a prompt?
                 writeError(ERR_DEBUG_MODULE, "Found a password prompt already...");
                 nFoundPrompt = PROMPT_PASSWORD;
                 
-                if (login->psServer->iLoginsDone < 1 && login->iId == 0)
-                  writeVerbose(VB_NONE_FILE, "Password Prompt Only: %s\n", login->psServer->pHostIP);
+                if (_psLogin->psServer->iLoginsDone < 1 && _psLogin->iId == 0)
+                  writeVerbose(VB_NONE_FILE, "Password Prompt Only: %s\n", _psLogin->psServer->pHostIP);
                 
                 break;
               }
@@ -356,9 +348,8 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
       if (nFoundPrompt == PROMPT_UNKNOWN)
       {
         writeError(ERR_ERROR, "[%s] Failed to identify logon prompt.", MODULE_NAME); 
-        login->iResult = LOGIN_RESULT_UNKNOWN;
-        login->iStatus = LOGIN_FAILED;
-        setPassResult(login, pPass);
+        _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+        setPassResult(_psLogin, psCredSet->pPass);
         return FAILURE;
       }
       else
@@ -368,15 +359,11 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
 
     case MSTATE_RUNNING:
       if (_psSessionData->nMode == MODE_AS400)
-        nState = tryLoginAS400(hSocket, &login, _psSessionData, user->pUser, pPass);
+        nState = tryLoginAS400(hSocket, &_psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
       else
-        nState = tryLogin(hSocket, &login, _psSessionData, user->pUser, pPass, nFoundPrompt);
+        nState = tryLogin(hSocket, &_psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass, nFoundPrompt);
       
-      if (nState == MSTATE_EXITING)
-      {
-        pPass = NULL;
-      }
-      else
+      if (nState != MSTATE_EXITING)
       {
         if (processIAClogout(hSocket, _psSessionData) == FAILURE)
         {
@@ -392,15 +379,25 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
         */
         sleep(1);
 
-        pPass = getNextPass(login->psServer->psAudit, user);
-        if (pPass == NULL)
+        if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
         {
-          writeVerbose(VB_GENERAL, "Done with passwords, exiting telnet");
+          writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
           nState = MSTATE_EXITING;
         }
         else
         {
-          nState = MSTATE_NEW;
+          if (psCredSet->iStatus == CREDENTIAL_DONE)
+          {
+            writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+          {
+            writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+            nState = MSTATE_NEW;
+          }
+          else
+            writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
         }
       }
       break;
@@ -408,15 +405,15 @@ int initModule(sLogin* login, _MODULE_DATA *_psSessionData)
       if (hSocket > 0)
         medusaDisconnect(hSocket);
       hSocket = -1;
+      nState = MSTATE_COMPLETE;
+      break;
     default:
-      writeError(ERR_CRITICAL, "Unknown %s module (%d) state %d host: %s", MODULE_NAME, login->iId, nState, login->psServer->pHostIP);
-      login->iResult = LOGIN_RESULT_UNKNOWN;
-      login->iStatus = LOGIN_FAILED;
-      setPassResult(login, pPass);
+      writeError(ERR_CRITICAL, "Unknown %s module (%d) state %d host: %s", MODULE_NAME, _psLogin->iId, nState, _psLogin->psServer->pHostIP);
+      _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
     }
   }
 
-  login->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -481,7 +478,7 @@ int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* sz
       // Are we at a known password prompt?
       for (i = 0; i < KNOWN_PWD_SIZE; i++)
       {
-        if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != NULL)
+        if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != '\0')
         {
           nContinue = 1;
           break;
@@ -493,7 +490,7 @@ int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* sz
       {
         for (i = 0; i < KNOWN_LOGIN_SIZE; i++)
         {
-          if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != NULL)
+          if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != '\0')
           {
             free(bufReceive);
             (*login)->iResult = LOGIN_RESULT_FAIL;
@@ -555,14 +552,14 @@ int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* sz
     /* check for known failures */
     if (strstr(bufReceive, "Authentication failed"))
     {
-      writeError(ERR_DEBUG, "Server responded with Cisco \"Authentication failed.\" message.");
+      writeError(ERR_DEBUG_MODULE, "Server responded with Cisco \"Authentication failed.\" message.");
       (*login)->iResult = LOGIN_RESULT_FAIL;
       setPassResult(*login, szPassword);
       return MSTATE_NEW;
     }
     if (strstr(bufReceive, "Login invalid"))
     {
-      writeError(ERR_DEBUG, "Server responded with Cisco \"Login invalid\" message.");
+      writeError(ERR_DEBUG_MODULE, "Server responded with Cisco \"Login invalid\" message.");
       (*login)->iResult = LOGIN_RESULT_FAIL;
       setPassResult(*login, szPassword);
       return MSTATE_NEW;
@@ -581,7 +578,7 @@ int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* sz
         // If we have a login prompt, we have failed
         for (i = 0; i < KNOWN_LOGIN_SIZE; i++)
         {
-          if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != NULL)
+          if (strcasestr(bufReceive, KNOWN_LOGIN_PROMPTS[i]) != '\0')
           {
             free(bufReceive);
             writeError(ERR_DEBUG_MODULE, "unsuccessful login - user '%s' with a password of '%s'", szLogin, szPassword);
@@ -596,7 +593,7 @@ int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* sz
         // We are operating on a non-login telnet setup
         for (i = 0; i < KNOWN_PWD_SIZE; i++)
         {
-          if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != NULL)
+          if (strcasestr(bufReceive, KNOWN_PWD_PROMPTS[i]) != '\0')
           {
             free(bufReceive);
             writeError(ERR_DEBUG_MODULE, "unsuccessful login with a password of '%s'", szPassword);
