@@ -2,7 +2,7 @@
 **   VMware Authentication Daemon Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -31,30 +31,28 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include "module.h"
 
 #define MODULE_NAME    "vmauthd.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for the VMware Authentication Daemon"
-#define MODULE_VERSION    "1.0.1"
-#define MODULE_VERSION_SVN "$Id: vmauthd.c 924 2008-05-08 14:49:51Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: vmauthd.c 1225 2009-10-13 18:55:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
 #define PORT_VMAUTHD 902
 
 typedef struct __VMAUTHD_DATA {
-  char *szDB;
 } _VMAUTHD_DATA;
 
-
 // Tells us whether we are to continue processing or not
 enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -85,7 +83,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -117,50 +114,49 @@ int go(sLogin* logins, int argc, char *argv[])
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
-
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _VMAUTHD_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = psLogin->psServer->psAudit->iPortOverride;
   else
     params.nPort = PORT_VMAUTHD;
-  params.nSSLVersion = 3; /* VMware Authentication Daemon requires SSLv3 */
-  initConnectionParams(psLogin, &params);
+   params.nSSLVersion = 3; /* VMware Authentication Daemon requires SSLv3 */
+   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         if (hSocket > 0)
@@ -171,14 +167,12 @@ int initModule(sLogin* psLogin, _VMAUTHD_DATA *_psSessionData)
         {
           writeError(ERR_NOTICE, "[%s] failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
         if (initConnection(hSocket, &params) == FAILURE)
         {
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           return FAILURE;
         }
 
@@ -187,15 +181,37 @@ int initModule(sLogin* psLogin, _VMAUTHD_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, user->pUser, pPass);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -203,12 +219,11 @@ int initModule(sLogin* psLogin, _VMAUTHD_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  psLogin->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
   return SUCCESS;
 }
        
@@ -221,32 +236,30 @@ int initModule(sLogin* psLogin, _VMAUTHD_DATA *_psSessionData)
 int initConnection(int hSocket, sConnectParams *params)
 { 
   int iRet;
-  unsigned char* bufReceive;
+  unsigned char* bufReceive = NULL;
   int nReceiveBufferSize;
   
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Incorrect VMware authentication protocol or service shutdown: %s.", MODULE_NAME, bufReceive);
+    FREE(bufReceive);
     return FAILURE;
   }
-  else if (strncmp(bufReceive, "220 VMware Authentication Daemon Version", 4) != 0)
-  {
-    writeError(ERR_ERROR, "[%s] Incorrect VMware authentication protocol or service shutdown: %s.", bufReceive);
-    return FAILURE;
-  }
-  
+ 
   if (strstr(bufReceive, "SSL Required"))
   {
     writeError(ERR_INFO, "[%s] SSL Required for VMware authentication. Proceeding using SSL.", MODULE_NAME);
-          
+  
     if (medusaConnectSocketSSL(params, hSocket) < 0)
     {
       writeError(ERR_ERROR, "[%s] Failed to establish SSLv3 connection.", MODULE_NAME);
+      FREE(bufReceive);
       return FAILURE;
     }
   }
+  
+  FREE(bufReceive);
 
   return SUCCESS;
 }
@@ -257,7 +270,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _VMAUTHD_DATA* _psSessionData, char*
   unsigned char* bufSend = NULL;
   unsigned char* bufReceive = NULL;
   int nReceiveBufferSize = 0;
-
+  
   bufSend = malloc(strlen(szLogin) + 7 + 1);
   memset(bufSend, 0, strlen(szLogin) + 7 + 1);
   sprintf(bufSend, "USER %s\r\n", szLogin);
@@ -265,27 +278,21 @@ int tryLogin(int hSocket, sLogin** psLogin, _VMAUTHD_DATA* _psSessionData, char*
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
-    free(bufSend);
+    FREE(bufSend);
     return FAILURE;
   }
-  free(bufSend);
+  FREE(bufSend);
 
-  nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
-  {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
-    return FAILURE;
-  }
   /* 331 Password required for USERNAME.[0D][0A] */
-  else if (strncmp(bufReceive, "331 ", 4) != 0)
+  nReceiveBufferSize = 0;
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "331 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] VMware authentication protocol error or service shutdown: %s.\n", bufReceive);
-    free(bufReceive);
+    writeError(ERR_ERROR, "[%s] VMware authentication protocol error or service shutdown: %s.\n", MODULE_NAME, bufReceive);
+    FREE(bufReceive);
     return FAILURE;
   }
-  free(bufReceive);
-
+  FREE(bufReceive);
+  
   bufSend = malloc(strlen(szPassword) + 7 + 1);
   memset(bufSend, 0, strlen(szPassword) + 7 + 1);
   sprintf(bufSend, "PASS %s\r\n", szPassword);
@@ -293,20 +300,21 @@ int tryLogin(int hSocket, sLogin** psLogin, _VMAUTHD_DATA* _psSessionData, char*
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
-    free(bufSend);
+    FREE(bufSend);
     return FAILURE;
   }
-  free(bufSend);
+  FREE(bufSend);
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^\\d+ .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] VMware authentication protocol error or service shutdown: %s.\n", MODULE_NAME, bufReceive);
+    FREE(bufReceive);
     return FAILURE;
   }
+
   /* 230 User USERNAME logged in.[0D][0A] */
-  else if (strncmp(bufReceive, "230 ", 4) == 0)
+  if (strncmp(bufReceive, "230 ", 4) == 0)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt successful.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
@@ -325,7 +333,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _VMAUTHD_DATA* _psSessionData, char*
     (*psLogin)->iResult = LOGIN_RESULT_ERROR;
     iRet = MSTATE_EXITING;
   }
-  free(bufReceive);
+  FREE(bufReceive);
   
   setPassResult((*psLogin), szPassword);
 
