@@ -2,7 +2,7 @@
 **   VNC Password Checking Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -40,8 +40,8 @@
 #define MODULE_NAME    "vnc.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for VNC sessions"
-#define MODULE_VERSION    "1.0.1"
-#define MODULE_VERSION_SVN "$Id: vnc.c 629 2006-11-30 21:37:03Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: vnc.c 1225 2009-10-13 18:55:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define PORT_VNC 5900
@@ -62,7 +62,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -93,7 +94,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -138,7 +138,7 @@ int go(sLogin* logins, int argc, char *argv[])
   } 
   else 
   {
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
  
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
@@ -167,22 +167,38 @@ int go(sLogin* logins, int argc, char *argv[])
     initModule(logins, psSessionData);
   }  
 
-  return 0;
+  FREE(psSessionData);
+  return SUCCESS;
 }
 
 int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
-  int i = 0, iRet;
-  char *pPass;
+  int iRet;
   unsigned char zChallenge[16];
-  sUser* user = psLogin->psUser;
   sConnectParams params;
   int nAngrySleep = 10;
   int bAuthAllowed = FALSE;
+  sCredentialSet *psCredSet = NULL;
+
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
 
   memset(&params, 0, sizeof(sConnectParams));
   if (psLogin->psServer->psAudit->iPortOverride > 0)
@@ -191,24 +207,9 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
     params.nPort = PORT_VNC;
   initConnectionParams(psLogin, &params);
 
-  if (user != NULL) 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
-  }
-  else 
-  {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
-  }
-
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         while (!bAuthAllowed)
@@ -222,22 +223,21 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
           {
             writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
             psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-            psLogin->iStatus = LOGIN_FAILED;
             return FAILURE;
           }
 
-          writeError(ERR_DEBUG, "Connected");
+          writeError(ERR_DEBUG_MODULE, "Connected");
 
           iRet = vncSessionSetup(hSocket, (unsigned char*) &zChallenge);
           switch( iRet )
           {
             case SESSION_SUCCESS:
-              writeError(ERR_DEBUG, "VNC Session Initialized.");
+              writeError(ERR_DEBUG_MODULE, "VNC Session Initialized.");
               bAuthAllowed = TRUE;
               nState = MSTATE_RUNNING;
               break;
             case SESSION_SUCCESS_NO_AUTH:
-              writeError(ERR_DEBUG, "VNC Server Does Not Require Authentication.");
+              writeError(ERR_DEBUG_MODULE, "VNC Server Does Not Require Authentication.");
               psLogin->iResult = LOGIN_RESULT_SUCCESS;
               setPassResult(psLogin, "[NO AUTH REQUIRED]");
               bAuthAllowed = TRUE;
@@ -249,7 +249,6 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
               {
                 writeError(ERR_ERROR, "[%s] Host %s exceeded maximum allowed sleep. Terminating connection.", MODULE_NAME, psLogin->psServer->pHostIP);
                 psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-                psLogin->iStatus = LOGIN_FAILED;
                 bAuthAllowed = TRUE;
                 nState = MSTATE_EXITING;
               }
@@ -265,7 +264,6 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
               {
                 writeError(ERR_ERROR, "[%s] Host %s exceeded maximum allowed sleep. Terminating connection.", MODULE_NAME, psLogin->psServer->pHostIP);
                 psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-                psLogin->iStatus = LOGIN_FAILED;
                 bAuthAllowed = TRUE;
                 nState = MSTATE_EXITING;
               }
@@ -276,9 +274,8 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
               }
               break;
             default:
-              writeError(ERR_DEBUG, "VNC Session Setup Failed.");
+              writeError(ERR_DEBUG_MODULE, "VNC Session Setup Failed.");
               psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-              psLogin->iStatus = LOGIN_FAILED;
               bAuthAllowed = TRUE;
               nState = MSTATE_EXITING;
               break;
@@ -288,15 +285,37 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
         bAuthAllowed = FALSE;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, (unsigned char*) &zChallenge, pPass);
+        nState = tryLogin(hSocket, &psLogin, _psSessionData, (unsigned char*) &zChallenge, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
           medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
@@ -304,15 +323,11 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
           medusaDisconnect(hSocket);
         hSocket = -1;
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
 
-  if (psLogin->iStatus != LOGIN_FAILED)
-     psLogin->iStatus = LOGIN_DONE;
-        
-  free(_psSessionData);
+  FREE(psCredSet);
   return SUCCESS;
 }
 
@@ -357,14 +372,14 @@ int vncSessionSetup(int hSocket, unsigned char* pzChallenge)
   if (bufReceive == NULL)
     return SESSION_FAILURE;
 
-  writeError(ERR_DEBUG, "VNC Server Protocol Version: %s", bufReceive);
+  writeError(ERR_DEBUG_MODULE, "VNC Server Protocol Version: %s", bufReceive);
 
   /* The following message is triggered by 5 failed authentication attempts, at which 
   ** point a 10 second lockout is applied before the next attempt is permitted.  The
   ** next failed attempt causes the timeout to be doubled. */
   if ((strncmp(bufReceive + 20, "Too many security failures", 10) == 0) || (strncmp(bufReceive + 20, "Too many authentication failures", 10) == 0))
   {
-    writeError(ERR_DEBUG, "[%s] Host reported too many security failures.", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "[%s] Host reported too many security failures.", MODULE_NAME);
     return SESSION_MAX_AUTH_REALVNC;
   }
   /*
@@ -385,15 +400,23 @@ int vncSessionSetup(int hSocket, unsigned char* pzChallenge)
     memcpy(ProtocolVersion, "RFB 003.007\n", 12);
     iServerProtocolVersion = 7;
   }
+  /*
+
+  */
+  else if (strncmp(bufReceive, "RFB 004.000", 11) == 0)
+  {
+    writeError(ERR_DEBUG_MODULE, "Unsupported VNC verions (RFB 004.000).");
+    return SESSION_FAILURE;
+  }
   else
   {
-    writeError(ERR_DEBUG, "[%s] Unknown session setup response: %s. Setting client response to version 3.", MODULE_NAME, bufReceive);
+    writeError(ERR_DEBUG_MODULE, "[%s] Unknown session setup response: %s. Setting client response to version 3.", MODULE_NAME, bufReceive);
     memcpy(ProtocolVersion, "RFB 003.003\n", 12);
     iServerProtocolVersion = 3;
   }
 
   /* Send client VNC protocol version */
-  writeError(ERR_DEBUG, "VNC Client Protocol Version: %s", ProtocolVersion);
+  writeError(ERR_DEBUG_MODULE, "VNC Client Protocol Version: %s", ProtocolVersion);
   if (medusaSend(hSocket, ProtocolVersion, 12, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
@@ -407,7 +430,7 @@ int vncSessionSetup(int hSocket, unsigned char* pzChallenge)
   bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
   if ((bufReceive == NULL) || (nReceiveBufferSize < 4))
   {
-    writeError(ERR_DEBUG, "Invalid session response (%d): %s.", nReceiveBufferSize, bufReceive);
+    writeError(ERR_DEBUG_MODULE, "Invalid session response (%d): %s.", nReceiveBufferSize, bufReceive);
     return SESSION_FAILURE;
   }
   else
@@ -415,20 +438,20 @@ int vncSessionSetup(int hSocket, unsigned char* pzChallenge)
     switch (bufReceive[3])
     {
       case 0x00:  /* connection failure */
-        writeError(ERR_DEBUG, "VNC Session Setup - Failed.");
+        writeError(ERR_DEBUG_MODULE, "VNC Session Setup - Failed.");
         /* Server is probably in anti-brute force mode (UltraVNC) */
         return SESSION_MAX_AUTH_ULTRAVNC;
         break;
       case 0x01:  /* no authentication required */
-        writeError(ERR_DEBUG, "VNC Session Setup - Successful - No Authentication Required.");
+        writeError(ERR_DEBUG_MODULE, "VNC Session Setup - Successful - No Authentication Required.");
         return SESSION_SUCCESS_NO_AUTH;
         break;
       case 0x02:  /* authentication required -- set authentication challenge */
-        writeError(ERR_DEBUG, "VNC Session Setup - Successful.");
+        writeError(ERR_DEBUG_MODULE, "VNC Session Setup - Successful.");
         if (nReceiveBufferSize == 20)
         {
           memcpy(pzChallenge, bufReceive + 4, 16);
-          writeError(ERR_DEBUG, "VNC authentication challenge: %s", pzChallenge);
+          writeError(ERR_DEBUG_MODULE, "VNC authentication challenge: %s", pzChallenge);
           return SESSION_SUCCESS;
         }
         else
@@ -453,9 +476,9 @@ int tryLogin(int hSocket, sLogin** psLogin, _VNC_DATA* _psSessionData, unsigned
   int nReceiveBufferSize = 0;
   int iRet;
 
-  writeError(ERR_DEBUG, "[%s] VNC authentication challenge: %s", MODULE_NAME, pzChallenge);
+  writeError(ERR_DEBUG_MODULE, "[%s] VNC authentication challenge: %s", MODULE_NAME, pzChallenge);
   vncEncryptBytes(pzChallenge, szPassword);
-  writeError(ERR_DEBUG, "[%s] VNC authentication response: %s", MODULE_NAME, pzChallenge);
+  writeError(ERR_DEBUG_MODULE, "[%s] VNC authentication response: %s", MODULE_NAME, pzChallenge);
 
   if (medusaSend(hSocket, pzChallenge, 16, 0) < 0)
   {
@@ -471,12 +494,12 @@ int tryLogin(int hSocket, sLogin** psLogin, _VNC_DATA* _psSessionData, unsigned
   switch (bufReceive[3])
   {
     case 0x00:
-      writeError(ERR_DEBUG, "[%s] VNC Authentication - Success", MODULE_NAME);
+      writeError(ERR_DEBUG_MODULE, "[%s] VNC Authentication - Success", MODULE_NAME);
       (*psLogin)->iResult = LOGIN_RESULT_SUCCESS; 
       iRet = MSTATE_EXITING;
       break;
     case 0x01:
-      writeError(ERR_DEBUG, "[%s] VNC Authentication - Failed", MODULE_NAME);
+      writeError(ERR_DEBUG_MODULE, "[%s] VNC Authentication - Failed", MODULE_NAME);
       (*psLogin)->iResult = LOGIN_RESULT_FAIL;
       iRet = MSTATE_NEW;
       break;
