@@ -30,23 +30,16 @@
  ***************************************************************************/
 
 #include "module.h"
-#include "math.h"
 
 #define MODULE_NAME    "web-form.mod"
 #define MODULE_AUTHOR  "Luciano Bello <luciano@linux.org.ar>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for web forms"
-#define MODULE_VERSION    "1.0.0"
-#define MODULE_VERSION_SVN "$Id: web-form.c 980 2008-10-01 20:22:51Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: web-form.c 1294 2010-02-04 19:03:49Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #ifdef HAVE_LIBSSL
 
 #define HTTP_PORT 80
@@ -69,16 +62,17 @@ typedef struct __MODULE_DATA {
 } _MODULE_DATA;
 
 // Tells us whether we are to continue processing or not
-enum HTTP_STATE
+enum MODULE_STATE
 {
-  HSTATE_NEW,
-  HSTATE_RUNNING,
-  HSTATE_EXITING
+  MSTATE_NEW,
+  MSTATE_RUNNING,
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
 int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort, char* szLogin, char* szPassword);
-int initHttp(_MODULE_DATA* _psSessionData, sLogin* login);
+int initModule(_MODULE_DATA* _psSessionData, sLogin* login);
 
 // Tell medusa how many parameters this module allows
 int getParamNumber()
@@ -143,7 +137,7 @@ int go(sLogin* logins, int argc, char *argv[])
   else
   {
     // Parameters are good - make module go now
-    writeError(ERR_DEBUG, "OMG teh %s module has been called!!", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
 
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
@@ -216,69 +210,69 @@ int go(sLogin* logins, int argc, char *argv[])
 
       free(pOptTmp);
     }
-    initHttp(psSessionData, logins);
+    initModule(psSessionData, logins);
   }
 
+  FREE(psSessionData);
   return SUCCESS;
 }
 
-int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
+int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
 {
   int hSocket = -1;
-  enum HTTP_STATE nState = HSTATE_NEW;
-  char* bufReceive;
-  int i = 0;
-  char *pPass;
+  enum MODULE_STATE nState = MSTATE_NEW;
   char *pStrtokSavePtr = NULL;
   char *pTemp;
-  int nPort, nBufLength = 0;
-  sUser* user = login->psUser;
+  int nBufLength = 0;
+  sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
+  {
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+  else if (psCredSet->psUser)
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, _psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
+  }
+
   memset(&params, 0, sizeof(sConnectParams));
-  if (login->psServer->psAudit->iPortOverride > 0)
-    params.nPort = login->psServer->psAudit->iPortOverride;
-  else if (login->psServer->psHost->iUseSSL > 0)
+  if (_psLogin->psServer->psAudit->iPortOverride > 0)
+    params.nPort = _psLogin->psServer->psAudit->iPortOverride;
+  else if (_psLogin->psServer->psHost->iUseSSL > 0)
     params.nPort = HTTPS_PORT;
   else
     params.nPort = HTTP_PORT; 
-  initConnectionParams(login, &params);
+  initConnectionParams(_psLogin, &params);
 
-  if (user != NULL)
+  while (nState != MSTATE_COMPLETE)
   {
-    writeError(ERR_DEBUG, "[%s] module started for host: %s:%d, user: '%s'", MODULE_NAME, login->psServer->pHostIP, params.nPort, user->pUser);
-  }
-  else
-  {
-    writeError(ERR_DEBUG, "null user");
-  }
-
-  pPass = getNextPass(login->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, login->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
-  {
-    switch(nState)
+    switch (nState)
     {
-      case HSTATE_NEW:
+      case MSTATE_NEW:
         // Already have an open socket - close it
         if (hSocket > 0)
           medusaDisconnect(hSocket);
 
-        if (login->psServer->psHost->iUseSSL > 0)
+        if (_psLogin->psServer->psHost->iUseSSL > 0)
           hSocket = medusaConnectSSL(&params);
         else
           hSocket = medusaConnect(&params);
 
         if (hSocket < 0)
         {
-          writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, login->psServer->pHostIP);
-          login->iResult = LOGIN_RESULT_UNKNOWN;
-          login->iStatus = LOGIN_DONE;
-          setPassResult(login, pPass);
+          writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, _psLogin->psServer->pHostIP);
+          _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+          setPassResult(_psLogin, psCredSet->pPass);
           return FAILURE;
         }
 
@@ -290,10 +284,10 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
         }
 
         if (!_psSessionData->szHostHeader) {
-          nBufLength = strlen(login->psServer->psHost->pHost) + 1 + log(params.nPort) + 1;
+          nBufLength = strlen(_psLogin->psServer->psHost->pHost) + 1 + log(params.nPort) + 1;
           _psSessionData->szHostHeader = malloc(nBufLength + 1);
           memset(_psSessionData->szHostHeader, 0, nBufLength + 1);
-          sprintf(_psSessionData->szHostHeader, "%s:%d", login->psServer->psHost->pHost, params.nPort);
+          sprintf(_psSessionData->szHostHeader, "%s:%d", _psLogin->psServer->psHost->pHost, params.nPort);
         }
 
         if (!_psSessionData->szFormData) {
@@ -382,32 +376,45 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
           sprintf(_psSessionData->szDenySignal, "Login Incorrect");
         }
 
-        nState = HSTATE_RUNNING;
+        nState = MSTATE_RUNNING;
         break;
-      case HSTATE_RUNNING:
-        nState = tryLogin(hSocket, _psSessionData, &login, params.nPort, user->pUser, pPass);
-        medusaDisconnect(hSocket);
+      case MSTATE_RUNNING:
+        nState = tryLogin(hSocket, _psSessionData, &_psLogin, params.nPort, psCredSet->psUser->pUser, psCredSet->pPass);
+
+        if (_psLogin->iResult != LOGIN_RESULT_UNKNOWN)
+        {
+          if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
+        break;
+      case MSTATE_EXITING:
+        if (hSocket > 0)
+          medusaDisconnect(hSocket);
         hSocket = -1;
-        pPass = getNextPass(login->psServer->psAudit, user);
-        if (pPass == NULL)
-        {
-          writeError(ERR_DEBUG_MODULE, "Done with passwords, exiting http");
-          nState = HSTATE_EXITING;
-        }
-        else
-        {
-          nState = HSTATE_NEW;
-        }
+        nState = MSTATE_COMPLETE;
+        break;
+      default:
+        writeError(ERR_CRITICAL, "Unknown HTTP module state (%d). Exiting...", nState);
+        _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
         break;
-    case HSTATE_EXITING:
-      if (hSocket > 0)
-        medusaDisconnect(hSocket);
-      hSocket = -1;
-    default:
-      writeError(ERR_CRITICAL, "Unknown HTTP module state (%d). Exiting...", nState);
-      login->iResult = LOGIN_RESULT_UNKNOWN;
-      login->iStatus = LOGIN_DONE;
-      setPassResult(login, pPass);
     }
   }
 
@@ -420,9 +427,8 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
   FREE(_psSessionData->szFormRest);
   FREE(_psSessionData->szFormUser);
   FREE(_psSessionData->szFormPass); 
-  FREE(_psSessionData);
-
-  login->iStatus = LOGIN_DONE;
+  FREE(psCredSet);
+  
   return SUCCESS;
 }
 
@@ -506,7 +512,7 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
     writeError(ERR_ERROR, "[%s] Failed during sending of authentication data.", MODULE_NAME);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;  
+    return MSTATE_EXITING;  
   }
 
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
@@ -517,16 +523,16 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
     writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;  
+    return MSTATE_EXITING;  
   }
 
-  pTemp = ((char*)index(pReceiveBuffer, ' ')) + 1;
-  if (strncmp(pTemp, "200 OK", 6) != 0 )
+  pTemp = (char*)index(pReceiveBuffer, ' ');
+  if ( !pTemp || strncmp(pTemp + 1, "200 OK", 6) != 0 )
   {
     writeError(ERR_ERROR, "The answer was NOT successfully received, understood, and accepted: error code %.4s", pTemp);
     (*login)->iResult = LOGIN_RESULT_UNKNOWN;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;
+    return MSTATE_EXITING;
   }
 
   while ((strcasestr(pReceiveBuffer, _psSessionData->szDenySignal) == NULL) && (pReceiveBuffer[0] != '\0'))
@@ -539,13 +545,13 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
   {
     (*login)->iResult = LOGIN_RESULT_FAIL;
     setPassResult(*login, szPassword);
-    return HSTATE_RUNNING;
+    return MSTATE_NEW;
   }
    
-  writeError(ERR_DEBUG, "Login Successful");
+  writeError(ERR_DEBUG_MODULE, "Login Successful");
   (*login)->iResult = LOGIN_RESULT_SUCCESS;
   setPassResult(*login, szPassword);
-  return HSTATE_RUNNING;   
+  return MSTATE_NEW;   
 }
 
 #else
@@ -573,7 +579,6 @@ void showUsage()
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
   writeVerbose(VB_NONE, "** Module was not properly built. Is OPENSSL installed correctly? **");
   writeVerbose(VB_NONE, "");
-  return FAILURE;
 }
 
 int go(sLogin* logins, int argc, char *argv[])
