@@ -2,7 +2,7 @@
 **   Generic Wrapper Medusa Module
 **
 **   ------------------------------------------------------------------------
-**    Copyright (C) 2006 Joe Mondloch
+**    Copyright (C) 2009 Joe Mondloch
 **    JoMo-Kun / jmk@foofus.net
 **
 **    This program is free software; you can redistribute it and/or modify
@@ -34,17 +34,11 @@
 #define MODULE_NAME    "wrapper.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Generic Wrapper Module"
-#define MODULE_VERSION    "1.0.1"
-#define MODULE_VERSION_SVN "$Id: wrapper.c 569 2006-07-31 21:01:06Z jmk $"
+#define MODULE_VERSION    "2.0"
+#define MODULE_VERSION_SVN "$Id: wrapper.c 1271 2009-11-25 00:24:58Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
-#define FREE(x) \
-        if (x != NULL) { \
-           free(x); \
-           x = NULL; \
-        }
-
 #define PARENT_READ   iReadPipe[0]
 #define CHILD_WRITE   iReadPipe[1]
 #define CHILD_READ    iWritePipe[0]
@@ -68,7 +62,8 @@ enum MODULE_STATE
 {
   MSTATE_NEW,
   MSTATE_RUNNING,
-  MSTATE_EXITING
+  MSTATE_EXITING,
+  MSTATE_COMPLETE
 };
 
 // Forward declarations
@@ -98,7 +93,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -205,34 +199,41 @@ int go(sLogin* logins, int argc, char *argv[])
     iRet = SUCCESS;
   }
 
+  FREE(psSessionData);
   return iRet;
 }
 
 int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
 {
   enum MODULE_STATE nState = MSTATE_NEW;
-  char *pPass, *szTmp, *szCmdTmp;
-  sUser* user = psLogin->psUser;
-  int iRet, nCmdLength, nCmdPartLength;
+  char *szTmp = NULL;
+  char *szCmdTmp = NULL;
+  int iRet, nCmdLength;
+  int64_t nCmdPartLength;
+  sCredentialSet *psCredSet = NULL;
+  sConnectParams params;
 
-  if (user != NULL) 
+  psCredSet = malloc( sizeof(sCredentialSet) );
+  memset(psCredSet, 0, sizeof(sCredentialSet));
+
+  if (getNextCredSet(psLogin, psCredSet) == FAILURE)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: '%s'", MODULE_NAME, psLogin->psServer->pHostIP, user->pUser);
+    writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
   }
-  else 
+  else if (psCredSet->psUser)
   {
-    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s", MODULE_NAME, psLogin->psServer->pHostIP);
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s user: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+  }
+  else
+  {
+    writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME);
+    nState = MSTATE_COMPLETE;
   }
 
-  pPass = getNextPass(psLogin->psServer->psAudit, user);
-  if (pPass == NULL)
-  {
-    writeVerbose(VB_GENERAL, "[%s] out of passwords for user '%s' at host '%s', bailing", MODULE_NAME, user->pUser, psLogin->psServer->pHostIP);
-  }
-
-  while(NULL != pPass)
+  while (nState != MSTATE_COMPLETE)
   {  
-    switch(nState)
+    switch (nState)
     {
       case MSTATE_NEW:
         /* perform parameter substitution -- this is messy... */
@@ -244,8 +245,8 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
           nCmdLength = strlen(_psSessionData->szCmdParam);
           nCmdLength -= 6;
           nCmdLength += strlen(psLogin->psServer->pHostIP);
-          nCmdLength += strlen(user->pUser);
-          nCmdLength += strlen(pPass);
+          nCmdLength += strlen(psCredSet->psUser->pUser);
+          nCmdLength += strlen(psCredSet->pPass);
         }
         /* --host %H --user %U */
         else
@@ -253,17 +254,19 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
           nCmdLength = strlen(_psSessionData->szCmdParam);
           nCmdLength -= 4;
           nCmdLength += strlen(psLogin->psServer->pHostIP);
-          nCmdLength += strlen(user->pUser);
+          nCmdLength += strlen(psCredSet->psUser->pUser);
         }
-        
-        _psSessionData->szCmdParamFull = malloc(nCmdLength + 1);
-        memset(_psSessionData->szCmdParamFull, 0, nCmdLength + 1);
-        szCmdTmp = malloc(nCmdLength + 1);
-        memset(szCmdTmp, 0, nCmdLength + 1);
+       
+        /* adding in 6 characters to temporarily account for %H %U %P values - needed if 
+           username/password are shorter than 2 characters each */
+        _psSessionData->szCmdParamFull = malloc(nCmdLength + 6 + 1);
+        memset(_psSessionData->szCmdParamFull, 0, nCmdLength + 6 + 1);
+        szCmdTmp = malloc(nCmdLength + 6 + 1);
+        memset(szCmdTmp, 0, nCmdLength + 6 + 1);
        
         if (szTmp = strstr(_psSessionData->szCmdParam, "%H"))
         {
-          nCmdPartLength = (int) szTmp - (int) _psSessionData->szCmdParam;
+          nCmdPartLength = (int64_t) szTmp - (int64_t) _psSessionData->szCmdParam;
           writeError(ERR_DEBUG_MODULE, "Processing \%H... Copying (%d) parameter characters.", nCmdPartLength);
           strncpy(szCmdTmp, _psSessionData->szCmdParam, nCmdPartLength); 
           strncpy(szCmdTmp + nCmdPartLength, psLogin->psServer->pHostIP, strlen(psLogin->psServer->pHostIP)); 
@@ -273,7 +276,6 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         {
           writeError(ERR_ERROR, "Invalid command parameter format. Missing %H format.");            
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           FREE(szCmdTmp);
           nState = MSTATE_EXITING;
           break;
@@ -283,17 +285,16 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         
         if (szTmp = strstr(szCmdTmp, "%U"))
         {
-          nCmdPartLength = (int) szTmp - (int) szCmdTmp;
+          nCmdPartLength = (int64_t) szTmp - (int64_t) szCmdTmp;
           writeError(ERR_DEBUG_MODULE, "Processing \%U... Copying (%d) parameter characters.", nCmdPartLength);
           strncpy(_psSessionData->szCmdParamFull, szCmdTmp, nCmdPartLength); 
-          strncpy(_psSessionData->szCmdParamFull + nCmdPartLength, user->pUser, strlen(user->pUser)); 
-          strncpy(_psSessionData->szCmdParamFull + nCmdPartLength + strlen(user->pUser), szTmp + 2, strlen(szTmp) - 2); 
+          strncpy(_psSessionData->szCmdParamFull + nCmdPartLength, psCredSet->psUser->pUser, strlen(psCredSet->psUser->pUser)); 
+          strncpy(_psSessionData->szCmdParamFull + nCmdPartLength + strlen(psCredSet->psUser->pUser), szTmp + 2, strlen(szTmp) - 2); 
         }
         else
         {
           writeError(ERR_ERROR, "Invalid command parameter format. Missing %U format.");
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           nState = MSTATE_EXITING;
           FREE(szCmdTmp);
           break;
@@ -303,11 +304,11 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
 
         if ((_psSessionData->nType == TYPE_SINGLE) && (szTmp = strstr(_psSessionData->szCmdParamFull, "%P")))
         {
-          nCmdPartLength = (int) szTmp - (int) _psSessionData->szCmdParamFull;
+          nCmdPartLength = (int64_t) szTmp - (int64_t) _psSessionData->szCmdParamFull;
           writeError(ERR_DEBUG_MODULE, "Processing \%P... Copying (%d) parameter characters.", nCmdPartLength);
           strncpy(szCmdTmp, _psSessionData->szCmdParamFull, nCmdPartLength); 
-          strncpy(szCmdTmp + nCmdPartLength, pPass, strlen(pPass)); 
-          strncpy(szCmdTmp + nCmdPartLength + strlen(pPass), szTmp + 2, strlen(szTmp) - 2); 
+          strncpy(szCmdTmp + nCmdPartLength, psCredSet->pPass, strlen(psCredSet->pPass)); 
+          strncpy(szCmdTmp + nCmdPartLength + strlen(psCredSet->pPass), szTmp + 2, strlen(szTmp) - 2); 
 
           strncpy(_psSessionData->szCmdParamFull, szCmdTmp, nCmdLength + 1);
         }
@@ -315,12 +316,14 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         {
           writeError(ERR_ERROR, "Invalid command parameter format. Missing %P format.");
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          psLogin->iStatus = LOGIN_FAILED;
           nState = MSTATE_EXITING;
           FREE(szCmdTmp);
           break;
         }
 
+        /* clean-up any extra characters left over from %H %U %P */
+        memset(_psSessionData->szCmdParamFull + nCmdLength, 0, 6 + 1);
+
         FREE(szCmdTmp);
         writeError(ERR_DEBUG_MODULE, "Parameters (pass 3): %s", _psSessionData->szCmdParamFull);
         
@@ -339,18 +342,39 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(_psSessionData, &psLogin, user->pUser, pPass);
+        nState = tryLogin(_psSessionData, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
+
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
-          pPass = getNextPass(psLogin->psServer->psAudit, user);
+        {
+          if (getNextCredSet(psLogin, psCredSet) == FAILURE)
+          {
+            writeError(ERR_ERROR, "[%s] Error retrieving next credential set to test.", MODULE_NAME);
+            nState = MSTATE_EXITING;
+          }
+          else
+          {
+            if (psCredSet->iStatus == CREDENTIAL_DONE)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] No more available credential sets to test.", MODULE_NAME);
+              nState = MSTATE_EXITING;
+            }
+            else if (psCredSet->iStatus == CREDENTIAL_NEW_USER)
+            {
+              writeError(ERR_DEBUG_MODULE, "[%s] Starting testing for new user: %s.", MODULE_NAME, psCredSet->psUser->pUser);
+              nState = MSTATE_NEW;
+            }
+            else
+              writeError(ERR_DEBUG_MODULE, "[%s] Next credential set - user: %s password: %s", MODULE_NAME, psCredSet->psUser->pUser, psCredSet->pPass);
+          }
+        }
         break;
       case MSTATE_EXITING:
         iRet = closeProcess(_psSessionData);
-        pPass = NULL;
+        nState = MSTATE_COMPLETE;
         break;
       default:
         writeError(ERR_CRITICAL, "Unknown %s module state %d", MODULE_NAME, nState);
         psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-        psLogin->iStatus = LOGIN_FAILED;
         return FAILURE;
     }  
   }
@@ -362,17 +386,9 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
   FREE(_psSessionData->szCmdFull);
   FREE(_psSessionData->szCmdParam);
   FREE(_psSessionData->szCmdParamFull);
-  FREE(_psSessionData);
+  FREE(psCredSet);
  
-  if (psLogin->iStatus == LOGIN_FAILED)
-  {
-    return FAILURE;
-  }
-  else
-  {
-    psLogin->iStatus = LOGIN_DONE;
-    return SUCCESS;
-  }
+  return SUCCESS;
 }
 
 /* Module Specific Functions */
@@ -439,7 +455,8 @@ int initProcess(_MODULE_DATA* _psSessionData, sLogin* psLogin)
       {
         /* launch application */
         //execlp(_psSessionData->szCmd, _psSessionData->szCmd, _psSessionData->szCmdParamFull, (char *)0);
-        system(_psSessionData->szCmdFull);
+        if (system(_psSessionData->szCmdFull) < 0)
+          writeError(ERR_ERROR, "[%s] system() call return error", MODULE_NAME);            
       }
      
       /* give the parent a chance to read the error message */
@@ -491,7 +508,6 @@ int tryLogin(_MODULE_DATA* _psSessionData, sLogin** psLogin, char* szLogin, char
     {
       writeError(ERR_ERROR, "Error writing to child process buffer.");
       (*psLogin)->iResult = LOGIN_RESULT_ERROR;
-      (*psLogin)->iStatus = LOGIN_FAILED;
       iRet = MSTATE_EXITING;
       nDone = 1;
     }
@@ -541,7 +557,7 @@ int tryLogin(_MODULE_DATA* _psSessionData, sLogin** psLogin, char* szLogin, char
         pBuf = strtok_r(pBufTmp, ":", &strtok_ptr);
         pBuf = strtok_r(NULL, "\n", &strtok_ptr);
         writeError(ERR_ERROR, "Child process responded with: LOGIN_RESULT_ERROR (%s).", pBuf);
-        free(pBufTmp);
+        FREE(pBufTmp);
         
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         iRet = MSTATE_EXITING;
